#ifndef _CAMERA_CAMERAPERSPECTIVE_H_
#define _CAMERA_CAMERAPERSPECTIVE_H_

#include "Camera/includes.h"
#include "Camera/CameraGenerique.h"

#ifdef USENAMESPACE
namespace Camera
{
#endif /* #ifdef USENAMESPACE */

/*! \class CameraPerspective
 * \brief class for perspective cameras objects
 * \author E. Guillou
 *
 * This object encapsulates internal parameters of a perspective camera (ie : focal length and aspect ratio, ...).
 * It inherits external parameters from CameraGenerique object.
 */

class CameraPerspective : public CameraGenerique {
private:
  /**
   * the focal length of the camera.
   */
  double _focallength;
  /**
   * the distortion factor of the camera.
   */
  double _distortion;
  /**
   * the aspect ratio of the camera.
   * ie : width/height of a pixel.
   */
  double _aspectratio;
  /**
   * the pixel size of the camera.
   * ie : width of a pixel in meters.
   */
  double _pixelsize;
  /**
   * the skew angle of the camera.
   */
  double _skewangle;
  /**
   * the principal point of the camera.
   */
  sPoint2d _principalpoint;
protected:
public:
  /**
   * Perspective camera default constructor.
   */
  CameraPerspective();
  /**
   * Perspective camera initialization constructor.
   */
  CameraPerspective(Coordinates_Definition);
  /**
   * Perspective camera copy constructor.
   */
  CameraPerspective(CameraPerspective&);
  /**
   * Perspective camera destructor.
   */
  ~CameraPerspective();
  /**
   * Perspective camera copy operator.
   */
  CameraPerspective& operator=(CameraPerspective&);
  /**
   * Perspective camera private members access method.
   * Access method to the focal length of the camera.
   */
  double& FocalLength();
  /**
   * Perspective camera private members access method.
   * Access method to the distortion factor of the camera.
   */
  double& Distortion();
  /**
   * Perspective camera private members access method.
   * Access method to the aspect ratio of the camera.
   */
  double& AspectRatio();
  /**
   * Perspective camera private members access method.
   * Access method to the pixel size of the camera.
   */
  double& PixelSize();
  /**
   * Perspective camera private members access method.
   * Access method to the skew angle of the camera.
   */
  double& SkewAngle();
  /**
   * Perspective camera private members access method.
   * Access method to the principal point of the camera.
   */
  sPoint2d& PrincipalPoint();
  /**
   * Perspective camera private members initialization method.
   * initialization method of the focal length of the camera.
   */
  void FocalLength(double);
  /**
   * Perspective camera private members initialization method.
   * initialization method of the distortion factor of the camera.
   */
  void Distortion(double);
  /**
   * Perspective camera private members initialization method.
   * initialization method of the aspect ratio of the camera.
   */
  void AspectRatio(double);
  /**
   * Perspective camera private members initialization method.
   * initialization method of the pixel size of the camera.
   */
  void PixelSize(double);
  /**
   * Perspective camera private members initialization method.
   * initialization method of the skew angle of the camera.
   */
  void SkewAngle(double);
  /**
   * Perspective camera private members initialization method.
   * initialization method of the principal point of the camera.
   */
  void PrincipalPoint(sPoint2d&);
  /**
   * access method to computed attributes.
   * access method to the direction vector associated to a 2d pixel.
   */
  sVector3d DirectionPixel(sPoint2d&);
  /**
   * access method to computed attributes.
   * access method to the position of a 3d point (in the image plane) associated to a 2d pixel.
   */
  sPoint3d PositionPixel(sPoint2d&);
  /**
   * access method to computed attributes.
   * access method to the field of view of the camera in respect to the height of the image plane.
   */
  double FieldofView(double);
}; /* class CameraPerspective */

#ifdef USENAMESPACE
} /* namespace Camera */
#endif /* #ifdef USENAMESPACE */

#endif /* #ifndef _CAMERA_CAMERAPERSPECTIVE_H_ */

