#ifndef _GEOMETRIC_DATA_H_
#define _GEOMETRIC_DATA_H_

#include "Geometric/Includes.h"

/** \defgroup structure2d structures for 2d datas. */
/** \defgroup structure3d structures for 3d datas. */
/** \defgroup structurealiases alias for 2d/3d datas. */
/** \defgroup structureallocator allocation methods for structure datas. */
/** \defgroup structuredesallocator desallocation methods for structure datas. */
/** \defgroup structuredumping dumping methods for structure datas. */

#ifdef USENAMESPACE
namespace Geometric
{
#endif /* #ifdef USENAMESPACE */

  /*! \struct __point2d
   * \brief a 2d point.
   * \ingroup structure2d
   * described by its x and y coordinates.
   */
  typedef struct __point2d
  {
    /** 2d coordinates of the point */
    sGeometricData x, y;
    /** default initializer of the point */
    __point2d() { x = 0.; y = 0.; };
    /** initializer of the point */
    __point2d(sGeometricData xx,sGeometricData yy) { x = xx; y = yy; };
  };
  /*! \struct __vector2d
   * \brief a 2d vector.
   * \ingroup structure2d
   * described by its x and y coordinates.
   */
  typedef struct __vector2d
  {
    /** 2d coordinates of the vector. */
    sGeometricData x, y;
    /** default initializer of the vector. */
    __vector2d() { x = 0.; y = 0.; };
    /** initializer of the vector. */
    __vector2d(sGeometricData xx,sGeometricData yy) { x = xx; y = yy; };
  };
  /*! \struct __line2d
   * \brief a 2d line.
   * \ingroup structure2d
   * described by a 2d point and a 2d vector.
   */
  typedef struct __line2d
  {
    /** a 2d point on the line. */
    __point2d p;
    /** the direction vector. */
    __vector2d v;
  };
  /*! \struct __polygon2d
   * \brief a 2d polygon.
   * \ingroup structure2d
   * described by the 2d points.
   */
  typedef struct __polygon2d
  {
    /** the 2d points describing the polygon. */
    __point2d pts[maximumNumberOfPointInPolygon];
    /** the real number of points describing the polygon. */
    int size;
  };
  /*! \struct __point3d
   * \brief a 3d point.
   * \ingroup structure3d
   * described by its x, y and z coordinates.
   */
  typedef struct __point3d
  {
    /** 3d coordinates of the point. */
    sGeometricData x, y, z;
    /** default initializer of the point. */
    __point3d() { x = 0.; y = 0.; z = 0.; };
    /** initializer of the point. */
    __point3d(sGeometricData xx,sGeometricData yy,sGeometricData zz) { x = xx; y = yy; z = zz; };
  };
  /*! \struct __vector3d
   * \brief a 3d vector.
   * \ingroup structure3d
   * described by its x, y and z coordinates.
   */
  typedef struct __vector3d
  {
    /** 3d coordinates of the vector. */
    sGeometricData x, y, z;
    /** default initializer of the vector. */
    __vector3d() { x = 0.; y = 0.; z = 0.; };
    /** initializer of the vector. */
    __vector3d(sGeometricData xx,sGeometricData yy,sGeometricData zz) { x = xx; y = yy; z = zz; };
  };
  /*! \struct __line3d
   * \brief a 3d line.
   * \ingroup structure3d
   * described by a 3d point and a 3d vector.
   */
  typedef struct __line3d
  {
    /** a 3d point on the line. */
    __point3d p;
    /** the direction vector. */
    __vector3d v;
  };
  /*! \struct __box3d
   * \brief a 3d bounding box.
   * \ingroup structure3d
   * described by two 3d points describing its extrema.
   * this bounding box is axis aligned.
   */
  typedef struct __box3d
  {
    /** the minimum values on each coordinate */
    __point3d min;
    /** the maximum values on each coordinate */
    __point3d max;
  };
  /*! \struct __plane3d
   * \brief a 3d plane.
   * \ingroup structure3d
   * described by a 3d point on the plane and a 3d normal vector to the plane.
   */
  typedef struct __plane3d
  {
    /** a 3d point on the plane. */
    __point3d p;
    /** the normal vector to the plane. */
    __vector3d v;
    /** the plane parameter (ie : d in the plane equation). */
    sGeometricData planeparameter;
  };
  /*! \struct __polygon3d
   * \brief a 3d polygon.
   * \ingroup structure3d
   * described by the 3d points.
   */
  typedef struct __polygon3d
  {
    /** the 3d points describing the polygon. */
    __point3d pts[maximumNumberOfPointInPolygon];
    /** the real number of points describing the polygon. */
    int size;
    /** the plane containing the polygon. */
    __plane3d plane;
    //@{
    /** some acceleration datas for intersection methods. */
    __vector3d ei[maximumNumberOfPointInPolygon];
    sGeometricData viei[maximumNumberOfPointInPolygon];
    int coord;
    __point2d aibi[maximumNumberOfPointInPolygon];
    sGeometricData ci[maximumNumberOfPointInPolygon];
    //@}
  };
  /*! \typedef __point2d sPoint2d
   * \brief a 2d point.
   * \ingroup structurealiases
   * an alias on __point2d.
   */
  typedef __point2d sPoint2d;
  /*! \typedef __vector2d sVector2d
   * \brief a 2d vector.
   * \ingroup structurealiases
   * an alias on __vector2d.
   */
  typedef __vector2d sVector2d;
  /*! \typedef __line2d sLine2d
   * \brief a 2d line.
   * \ingroup structurealiases
   * an alias on __line2d.
   */
  typedef __line2d sLine2d;
  /*! \typedef __polygon2d sPolygon2d
   * \brief a 2d polygon.
   * \ingroup structurealiases
   * an alias on __polygon2d.
   */
  typedef __polygon2d sPolygon2d;
  /*! \typedef __point3d sPoint3d
   * \brief a 3d point.
   * \ingroup structurealiases
   * an alias on __point3d.
   */
  typedef __point3d sPoint3d;
  /*! \typedef __vector3d sVector3d
   * \brief a 3d vector.
   * \ingroup structurealiases
   * an alias on __vector3d.
   */
  typedef __vector3d sVector3d;
  /*! \typedef __line3d sLine3d
   * \brief a 3d line.
   * \ingroup structurealiases
   * an alias on __line3d.
   */
  typedef __line3d sLine3d;
  /*! \typedef __box3d sBox3d
   * \brief a 3d bounding box.
   * \ingroup structurealiases
   * an alias on __box3d.
   */
  typedef __box3d sBox3d;
  /*! \typedef __plane3d sPlane3d
   * \brief a 3d plane.
   * \ingroup structurealiases
   * an alias on __plane3d.
   */
  typedef __plane3d sPlane3d;
  /*! \typedef __polygon3d sPolygon3d
   * \brief a 3d polygon.
   * \ingroup structurealiases
   * an alias on __polygon3d.
   */
  typedef __polygon3d sPolygon3d;

  /** \fn sPoint2d* createPoint2d()
   * \ingroup structureallocator
   * \return pointer on a 2d point structure.
   * \brief methods to allocate 2d point in memory.
   */
  extern sPoint2d* createPoint2d();
  /** \fn sVector2d* createVector2d()
   * \ingroup structureallocator
   * \return pointer on a 2d vector structure.
   * \brief methods to allocate 2d vector in memory.
   */
  extern sVector2d* createVector2d();
  /** \fn sLine2d* createLine2d()
   * \ingroup structureallocator
   * \return pointer on a 2d line structure.
   * \brief methods to allocate 2d line in memory.
   */
  extern sLine2d* createLine2d();
  /** \fn sPolygon2d* createPolygon2d()
   * \ingroup structureallocator
   * \return pointer on a 2d polygon structure.
   * \brief methods to allocate 2d polygon in memory.
   */
  extern sPolygon2d* createPolygon2d();
  /** \fn sVector3d* createPoint3d()
   * \ingroup structureallocator
   * \return pointer on a 3d point structure.
   * \brief methods to allocate 3d point in memory.
   */
  extern sPoint3d* createPoint3d();
  /** \fn sVector3d* createVector3d()
   * \ingroup structureallocator
   * \return pointer on a 3d vector structure.
   * \brief methods to allocate 3d vector in memory.
   */
  extern sVector3d* createVector3d();
  /** \fn sLine3d* createLine3d()
   * \ingroup structureallocator
   * \return pointer on a 3d line structure.
   * \brief methods to allocate 3d line in memory.
   */
  extern sLine3d* createLine3d();
  /** \fn sBox3d* createBox3d()
   * \ingroup structureallocator
   * \return pointer on a 3d bounding box structure.
   * \brief methods to allocate 3d bounding box in memory.
   */
  extern sBox3d* createBox3d();
  /** \fn sPlane3d* createPlane3d()
   * \ingroup structureallocator
   * \return pointer on a 3d plane structure.
   * \brief methods to allocate 3d plane in memory.
   */
  extern sPlane3d* createPlane3d();
  /** \fn sPolygon3d* createPolygon3d()
   * \ingroup structureallocator
   * \return pointer on a 3d polygon structure.
   * \brief methods to allocate 3d polygon in memory.
   */
  extern sPolygon3d* createPolygon3d();
  /** \fn void deletePoint2d(sPoint2d*& s)
   * \ingroup structuredesallocator
   * \param s sPoint2d*&, the reference on the pointer to free.
   * \brief methods to free 2d point from memory.
   * in return, the pointer is set to NULL.
   */
  extern void deletePoint2d(sPoint2d*& s);
  /** \fn void deleteVector2d(sVector2d*& s);
   * \ingroup structuredesallocator
   * \param s sVector2d*&, the reference on the pointer to free.
   * \brief methods to free 2d vector from memory.
   * in return, the pointer is set to NULL.
   */
  extern void deleteVector2d(sVector2d*& s);
  /** \fn void deleteLine2d(sLine2d*& s);
   * \ingroup structuredesallocator
   * \param s sLine2d*&, the reference on the pointer to free.
   * \brief methods to free 2d line from memory.
   * in return, the pointer is set to NULL.
   */
  extern void deleteLine2d(sLine2d*& s);
  /** \fn void deletePolygon2d(sPolygon2d*& s);
   * \ingroup structuredesallocator
   * \param s sPolygon2d*&, the reference on the pointer to free.
   * \brief methods to free 2d polygon from memory.
   * in return, the pointer is set to NULL.
   */
  extern void deletePolygon2d(sPolygon2d*& s);
  /** \fn void deletePoint3d(sPoint3d*& s);
   * \ingroup structuredesallocator
   * \param s sPoint3d*&, the reference on the pointer to free.
   * \brief methods to free 3d point from memory.
   * in return, the pointer is set to NULL.
   */
  extern void deletePoint3d(sPoint3d*& s);
  /** \fn void deleteVector3d(sVector3d*& s);
   * \ingroup structuredesallocator
   * \param s sVector3d*&, the reference on the pointer to free.
   * \brief methods to free 3d vector from memory.
   * in return, the pointer is set to NULL.
   */
  extern void deleteVector3d(sVector3d*& s);
  /** \fn void deleteLine3d(sLine3d*& s);
   * \ingroup structuredesallocator
   * \param s sLine3d*&, the reference on the pointer to free.
   * \brief methods to free 3d line from memory.
   * in return, the pointer is set to NULL.
   */
  extern void deleteLine3d(sLine3d*& s);
  /** \fn void deleteBox3d(sBox3d*& s);
   * \ingroup structuredesallocator
   * \param s sBox3d*&, the reference on the pointer to free.
   * \brief methods to free 3d bounding box from memory.
   * in return, the pointer is set to NULL.
   */
  extern void deleteBox3d(sBox3d*& s);
  /** \fn void deletePlane3d(sPlane3d*& s);
   * \ingroup structuredesallocator
   * \param s sPlane3d*&, the reference on the pointer to free.
   * \brief methods to free 3d plane from memory.
   * in return, the pointer is set to NULL.
   */
  extern void deletePlane3d(sPlane3d*& s);
  /** \fn void deletePolygon3d(sPolygon3d*& s);
   * \ingroup structuredesallocator
   * \param s sPolygon3d*&, the reference on the pointer to free.
   * \brief methods to free 3d polygon from memory.
   * in return, the pointer is set to NULL.
   */
  extern void deletePolygon3d(sPolygon3d*& s);
  /** \fn ostream& operator<<(ostream& out,const sPoint2d& p);
   * \ingroup structuredumping
   * \param out ostream&, the reference on the stream.
   * \param p sPoint2d&, the reference on the data to dump.
   * \return the reference on the stream.
   * methods to dump 2d point on stream.
   */
  extern ostream& operator<<(ostream& out,const sPoint2d& p);
  /** \fn ostream& operator<<(ostream& out,const sVector2d& v);
   * \ingroup structuredumping
   * \param out ostream&, the reference on the stream.
   * \param v sVector2d&, the reference on the data to dump.
   * \return the reference on the stream.
   * methods to dump 2d vector on stream.
   */
  extern ostream& operator<<(ostream& out,const sVector2d& v);
  /** \fn ostream& operator<<(ostream& out,const sLine2d& l);
   * \ingroup structuredumping
   * \param out ostream&, the reference on the stream.
   * \param l sLine2d&, the reference on the data to dump.
   * \return the reference on the stream.
   * methods to dump 2d line on stream.
   */
  extern ostream& operator<<(ostream& out,const sLine2d& l);
  /** \fn ostream& operator<<(ostream& out,const sPoint3d& p);
   * \ingroup structuredumping
   * \param out ostream&, the reference on the stream.
   * \param p sPoint3d&, the reference on the data to dump.
   * \return the reference on the stream.
   * methods to dump 3d point on stream.
   */
  extern ostream& operator<<(ostream& out,const sPoint3d& p);
  /** \fn ostream& operator<<(ostream& out,const sVector3d& v);
   * \ingroup structuredumping
   * \param out ostream&, the reference on the stream.
   * \param v sVector3d&, the reference on the data to dump.
   * \return the reference on the stream.
   * methods to dump 3d vector on stream.
   */
  extern ostream& operator<<(ostream& out,const sVector3d& v);
  /** \fn ostream& operator<<(ostream& out,const sLine3d& l);
   * \ingroup structuredumping
   * \param out ostream&, the reference on the stream.
   * \param l sLine3d&, the reference on the data to dump.
   * \return the reference on the stream.
   * methods to dump 3d line on stream.
   */
  extern ostream& operator<<(ostream& out,const sLine3d& l);
  /** \fn ostream& operator<<(ostream& out,const sPlane3d& l);
   * \ingroup structuredumping
   * \param out ostream&, the reference on the stream.
   * \param l sPlane3d&, the reference on the data to dump.
   * \return the reference on the stream.
   * methods to dump 3d plane on stream.
   */
  extern ostream& operator<<(ostream& out,const sPlane3d& l);
  /** \fn ostream& operator<<(ostream& out,const sPolygon3d& l);
   * \ingroup structuredumping
   * \param out ostream&, the reference on the stream.
   * \param l sPolygon3d&, the reference on the data to dump.
   * \return the reference on the stream.
   * methods to dump 3d polygon on stream.
   */
  extern ostream& operator<<(ostream& out,const sPolygon3d& l);

#ifdef USENAMESPACE
} /* namespace Geometric */
#endif /* #ifdef USENAMESPACE */

#endif /* #ifndef _GEOMETRIC_DATA_H_ */
