#ifndef _GEOMETRIC_FUNCTIONS_H_
#define _GEOMETRIC_FUNCTIONS_H_

#include "Geometric/Includes.h"

/** \defgroup structuremanipulation manipulation methods for structure datas. */

#ifdef USENAMESPACE
namespace Geometric
{
#endif /* #ifdef USENAMESPACE */

  //@{
  /** \name structure members access functions. */
  /** \def x(p)
   * \brief access method to the x coordinate.
   * \ingroup structuremanipulation
   * Macro usable on 2d/3d point/vector.
   */
#define x(p) ( \
  (p).x \
)
  /** \def y(p)
   * \brief access method to the y coordinate.
   * \ingroup structuremanipulation
   * Macro usable on 2d/3d point/vector.
   */
#define y(p) ( \
  (p).y \
)
  /** \def z(p)
   * \brief access method to the z coordinate.
   * \ingroup structuremanipulation
   * Macro usable on 3d point/vector.
   */
#define z(p) ( \
  (p).z \
)
  /** \def vertex(pp,ii)
   * \brief access to the ii'th vertex of a polygon.
   * \ingroup structuremanipulation
   * Macro usable with 2d/3d polygons.
   */
#define vertex(pp,ii) ( \
  (pp).pts[(ii)] \
)
  //@}
  //@{
  /** \name matrix manipulation functions. */
  /** \def det2(v0,v1)
   * \brief determinant of a 2x2 matrix defined by 2d column vectors v0 and v1.
   * \ingroup structuremanipulation
   */
#define det2(v0,v1) ( \
  (x(v0)*y(v1))-(x(v1)*y(v0)) \
)
  /** \def det2_3(v0,v1,i)
   * \brief determinant of a 2x2 matrix defined by 3d column vectors v0 and v1 (forget ith coordinate).
   * \ingroup structuremanipulation
   */
#define det2_3(v0,v1,i) ( \
 (i == 0 ? (y(v0)*z(v1))-(y(v1)*z(v0)) : \
 (i == 1 ? (x(v0)*z(v1))-(x(v1)*z(v0)) : \
 (x(v0)*y(v1))-(x(v1)*y(v0)))) \
)
  /** \def det3(v0,v1,v2)
   * \brief determinant of a 3x3 matrix defined by 3d column vectors v0, v1 and v2.
   * \ingroup structuremanipulation
   */
#define det3(v0,v1,v2) ( \
  (x(v0)*det2_3(v1,v2,0))-(y(v0)*det2_3(v1,v2,1))+(z(v0)*det2_3(v1,v2,2)) \
)
  //@}
  //@{
  /**
   * \name access methods to the coefficients of the 2d line equation : a.x + b.y + c = 0.
   * \ingroup structuremanipulation
   */
#define a(d) ( \
  y((d).v) \
)
#define b(d) ( \
  -x((d).v) \
)
#define c(d) ( \
  x((d).v)*y((d).p)-y((d).v)*x((d).p) \
)
  //@}
  //@{
  /**
   * \name access methods to the coefficients of the 2d line equation : y = A.x + B
   * \ingroup structuremanipulation
   */
#define A(d) ( \
  -a(d)/b(d) \
)
#define B(d) ( \
  -c(d)/b(d) \
)
  //@}
  //@{
  /**
   * \name access methods to the coefficients of the 2d line equation : x = Cy+D
   * \ingroup structuremanipulation
   */
#define C(d) ( \
  -b(d)/a(d) \
)
#define D(d) ( \
  -c(d)/a(d) \
)
  //@}
  //@{
  /**
   * \name access methods to the coefficients of the 3d plane equation : ap.x + bp.y + cp.z + dp = 0
   * \ingroup structuremanipulation
   */
#define ap(pp) ( \
  x((pp).v) \
)
#define bp(pp) ( \
  y((pp).v) \
)
#define cp(pp) ( \
  z((pp).v) \
)
#define dp(pp) ( \
  -scalarProductVector3d((pp).v,(pp).p) \
)
  //@}
  //@{
  /**
   * \name methods to orthogonaly project points/vectors on axis.
   * \ingroup structuremanipulation
   * Macro usable with 2d/3d points/vectors.
   * the coord parameter describes where to project :
   *   - 0, x axis.
   *   - 1, y axis.
   *   - 2, z axis (only for 3d datas).
   */
#define getu(pp,coord) ( \
  (coord) == 0 ? y(pp) : (coord) == 1 ? z(pp) : x(pp) \
)
#define getv(pp,coord) ( \
  (coord) == 0 ? z(pp) : (coord) == 1 ? x(pp) : y(pp) \
)
  //@}
  //@{
  /**
   * \name declaration and initialization of geometric datas.
   * \ingroup structuremanipulation
   */
#define point2d(p,xx,yy) \
  sPoint2d p; \
  initPoint2d(p,xx,yy);
#define vector2d(p,xx,yy) \
  sVector2d p; \
  initVector2d(p,xx,yy);
#define line2d(d,pp,vv) \
  sLine2d d; \
  initLine2d(d,pp,vv);
#define point3d(p,xx,yy,zz) \
  sPoint3d p; \
  initPoint3d(p,xx,yy,zz);
#define vector3d(p,xx,yy,zz) \
  sVector3d p; \
  initVector3d(p,xx,yy,zz);
#define line3d(d,pp,vv) \
  sLine3d d; \
  initLine3d(d,pp,vv);
#define box3d(b,p0,p1) \
  sBox3d b; \
  initBox3d(b,p0,p1);
#define plane3d(pp,p0,v0) \
  sPlane3d pp; \
  initPlane3d(pp,p0,v0);
  //@}
  //@{
  /**
   * \name initialization of geometric datas.
   * \ingroup structuremanipulation
   */
#define initPoint2d(v,xx,yy) { \
  (v).x = (xx); \
  (v).y = (yy); \
}
#define initVector2d(v,xx,yy) { \
  (v).x = (xx); \
  (v).y = (yy); \
}
#define initLine2d(d,pp,vv) { \
  copyPoint2d(pp,(d).p); \
  copyVector2d(vv,(d).v); \
  normalizeVector2d((d).v); \
}
#define initPoint3d(v,xx,yy,zz) { \
  (v).x = (xx); \
  (v).y = (yy); \
  (v).z = (zz); \
}
#define initVector3d(v,xx,yy,zz) { \
  (v).x = (xx); \
  (v).y = (yy); \
  (v).z = (zz); \
}
#define initLine3d(d,pp,vv) { \
  copyPoint3d(pp,(d).p); \
  copyVector3d(vv,(d).v); \
  normalizeVector3d((d).v); \
}
#define initBox3d(v,p0,p1) { \
  copyPoint3d(p0,(v).min); \
  copyPoint3d(p1,(v).max); \
}
#define initPlane3d(pp,p0,v0) { \
  copyPoint3d(p0,(pp).p); \
  copyVector3d(v0,(pp).v); \
  normalizeVector3d((pp).v); \
  (pp).planeparameter = dp(pp); \
}
  //@}
  //@{
  /**
   * \name copy of geometric datas.
   * \ingroup structuremanipulation
   */
#define copyPoint2d(ps,pd) { \
  initPoint2d(pd,(ps).x,(ps).y) \
}
#define copyVector2d(ps,pd) { \
  initVector2d(pd,(ps).x,(ps).y) \
}
#define copyLine2d(ds,dd) { \
  copyPoint2d((ds).p,(dd).p); \
  copyVector2d((ds).v,(dd).v); \
}
#define copyPoint3d(ps,pd) { \
  initPoint3d(pd,(ps).x,(ps).y,(ps).z) \
}
#define copyVector3d(ps,pd) { \
  initVector3d(pd,(ps).x,(ps).y,(ps).z) \
}
#define copyLine3d(ds,dd) { \
  copyPoint3d((ds).p,(dd).p); \
  copyVector3d((ds).v,(dd).v); \
}
#define copyBox3d(ps,pd) { \
  copyPoint3d((ps).min,(pd).min); \
  copyPoint3d((ps).max,(pd).max); \
}
#define copyPlane3d(ps,pd) { \
  copyPoint3d((ps).p,(pd).p); \
  copyVector3d((ps).v,(pd).v); \
  (p0).planeparameter = (p1).planeparameter; \
}
  //@}
  //@{
  /**
   * \name manipulation of 2d geometric datas.
   */
  /** \def addVectorPoint2d(vv,p0,p1)
   * \brief adds two 2d vectors/points.
   * \ingroup structuremanipulation
   * vv = p0 + p1
   */
#define addVectorPoint2d(vv,p0,p1) { \
  initVector2d(vv,(p1).x+(p0).x,(p1).y+(p0).y); \
}
  /** \def sousVectorPoint2d(vv,p0,p1)
   * \brief substract two 2d vectors/points.
   * \ingroup structuremanipulation
   * vv = p0 - p1
   */
#define sousVectorPoint2d(vv,p0,p1) { \
  initVector2d(vv,(p1).x-(p0).x,(p1).y-(p0).y); \
}
  /** \def mulVectorScalar2d(vv,f)
   * \brief multiply a 2d vector/point by a scalar.
   * \ingroup structuremanipulation
   * vv = f * vv
   */
#define mulVectorScalar2d(vv,f) { \
  initVector2d(vv,(vv).x*(f),(vv).y*(f)); \
}
  /** \def normVector2d(vv)
   * \brief returns the norm of a 2d vector/point.
   * \ingroup structuremanipulation
   */
#define normVector2d(vv) ( \
  sqrt((vv).x*(vv).x+(vv).y*(vv).y) \
)
  /** \def normalizeVector2d(vv)
   * \brief normalize a 2d vector/point.
   * \ingroup structuremanipulation
   */
#define normalizeVector2d(vv) { \
  sGeometricData _n = normVector2d(vv); \
  mulVectorScalar2d(vv,1./_n); \
}
  /** \def scalarProductVector2d(v0,v1)
   * \brief dot product of two 2d vectors/points.
   * \ingroup structuremanipulation
   */
#define scalarProductVector2d(v0,v1) ( \
  (v0).x*(v1).x+(v0).y*(v1).y \
)
  /** \def colinearVector2d(v0,v1)
   * \brief test if two 2d vectors are colinears.
   * \ingroup structuremanipulation
   */
#define colinearVector2d(v0,v1) ( \
  ABS(scalarProductVector2d(v0,v1)) > (1. - epsilonValue) \
)
  /** \def orthogonalVector2d(v0,v1)
   * \brief test if two 2d vectors are orthogonals.
   * \ingroup structuremanipulation
   */
#define orthogonalVector2d(v0,v1) ( \
  ABS(scalarProductVector2d(v0,v1)) < epsilonValue \
)
  /** \def parallelLine2d(d0,d1)
   * \brief test if two 2d lines are parallels.
   * \ingroup structuremanipulation
   */
#define parallelLine2d(d0,d1) ( \
  colinearVector2d((d0).v,(d1).v) \
)
  /** \def orthogonalLine2d(d0,d1)
   * \brief test if two 2d lines are orthogonals.
   * \ingroup structuremanipulation
   */
#define orthogonalLine2d(d0,d1) ( \
  orthogonalVector2d((d0).v,(d1).v) \
)
  //@}
  //@{
  /**
   * \name manipulation of 3d geometric datas.
   */
  /** \def addVectorPoint3d(vv,p0,p1)
   * \brief adds two 3d vectors/points.
   * \ingroup structuremanipulation
   * vv = p0 + p1
   */
#define addVectorPoint3d(vv,p0,p1) { \
  initVector3d(vv,(p1).x+(p0).x,(p1).y+(p0).y,(p1).z+(p0).z); \
}
  /** \def sousVectorPoint3d(vv,p0,p1)
   * \brief substract two 3d vectors/points.
   * \ingroup structuremanipulation
   * vv = p0 - p1
   */
#define sousVectorPoint3d(vv,p0,p1) { \
  initVector3d(vv,(p1).x-(p0).x,(p1).y-(p0).y,(p1).z-(p0).z); \
}
  /** \def mulVectorScalar3d(vv,f)
   * \brief multiply a 3d vector/point by a scalar.
   * \ingroup structuremanipulation
   * vv = f * vv
   */
#define mulVectorScalar3d(vv,f) { \
  initVector3d(vv,(vv).x*(f),(vv).y*(f),(vv).z*(f)); \
}
  /** \def normVector3d(vv)
   * \brief returns the norm of a 3d vector/point.
   * \ingroup structuremanipulation
   */
#define normVector3d(vv) ( \
  sqrt((vv).x*(vv).x+(vv).y*(vv).y+(vv).z*(vv).z) \
)
  /** \def normalizeVector3d(vv)
   * \brief normalize a 3d vector/point.
   * \ingroup structuremanipulation
   */
#define normalizeVector3d(vv) { \
  sGeometricData _n = normVector3d(vv); \
  mulVectorScalar3d(vv,1./_n); \
}
  /** \def scalarProductVector3d(v0,v1)
   * \brief dot product of two 3d vectors/points.
   * \ingroup structuremanipulation
   */
#define scalarProductVector3d(v0,v1) ( \
  (v0).x*(v1).x+(v0).y*(v1).y+(v0).z*(v1).z \
)
  /** \def crossProductVector3d(vv,v0,v1)
   * \brief cross product of two 3d vectors/points.
   * \ingroup structuremanipulation
   */
#define crossProductVector3d(vv,v0,v1) { \
  initVector3d(vv, \
    ((v0).y*(v1).z)-((v0).z*(v1).y),\
    ((v0).z*(v1).x)-((v0).x*(v1).z), \
    ((v0).x*(v1).y)-((v0).y*(v1).x) \
  ); \
}
  /** \def colinearVector3d(v0,v1)
   * \brief test if two 3d vectors are colinears.
   * \ingroup structuremanipulation
   */
#define colinearVector3d(v0,v1) ( \
  ABS(scalarProductVector3d(v0,v1)) > (1. - epsilonValue) \
)
  /** \def orthogonalVector3d(v0,v1)
   * \brief test if two 3d vectors are orthogonals.
   * \ingroup structuremanipulation
   */
#define orthogonalVector3d(v0,v1) ( \
  ABS(scalarProductVector3d(v0,v1)) < epsilonValue \
)
  /** \def parallelLine3d(d0,d1)
   * \brief test if two 3d lines are parallels.
   * \ingroup structuremanipulation
   */
#define parallelLine3d(d0,d1) ( \
  colinearVector3d((d0).v,(d1).v) \
)
  /** \def orthogonalLine3d(d0,d1)
   * \brief test if two 3d lines are orthogonals.
   * \ingroup structuremanipulation
   */
#define orthogonalLine3d(d0,d1) ( \
  orthogonalVector3d((d0).v,(d1).v) \
)
  //@}

#ifdef USENAMESPACE
} /* namespace Geometric */
#endif /* #ifdef USENAMESPACE */

#endif /* #ifndef _GEOMETRIC_FUNCTIONS_H_ */
