#ifndef _GEOMETRIC_INTERSECTFUNCTIONS_H_
#define _GEOMETRIC_INTERSECTFUNCTIONS_H_

#include "Geometric/Includes.h"
#include "Geometric/Functions.h"

/** \defgroup structureintersection intersection methods for structure datas. */

#ifdef USENAMESPACE
namespace Geometric
{
#endif /* #ifdef USENAMESPACE */

  /** \def parameterPointOnLine2d(pp,d)
   * \brief gets the 't' parameter of point pp on line d.
   * \ingroup structureintersection
   */
#define parameterPointOnLine2d(pp,d) ( \
  ABS(x((d).v)) < smallEpsilonValue ? \
    (ABS(y((d).v)) < smallEpsilonValue ? \
      0 : \
      (y(pp)-y((d).p))/y((d).v)) : \
    (x(pp)-x((d).p))/x((d).v) \
)
  /** \def parameterPointOnLine3d(pp,d)
   * \brief gets the 't' parameter of point pp on line d.
   * \ingroup structureintersection
   */
#define parameterPointOnLine3d(pp,d) ( \
  ABS(x((d).v)) < smallEpsilonValue ? \
    (ABS(y((d).v)) < smallEpsilonValue ? \
      (ABS(z((d).v)) < smallEpsilonValue ? \
        0 : \
        (z(pp)-z((d).p))/z((d).v)) : \
      (y(pp)-y((d).p))/y((d).v)) : \
    (x(pp)-x((d).p))/x((d).v) \
)
  /** \def parameterPointOnLineIntersectingPlane3d(pp,dd)
   * \brief gets the 't' parameter of a point at the intersection of plane pp and line dd.
   * \ingroup structureintersection
   */
#define parameterPointOnLineIntersectingPlane3d(pp,dd) ( \
  -((pp).planeparameter + scalarProductVector3d((pp).v,(dd).p))/scalarProductVector3d((pp).v,(dd).v) \
)
  /** \def pointOnLine2d(pp,d)
   * \brief tests if point pp is on line d.
   * \ingroup structureintersection
   */
#define pointOnLine2d(pp,d) ( \
  (x((d).v)*(y(pp)-y((d).p))) == (y((d).v)*(x(pp)-x((d).p))) \
)
  /** \def pointOnLine3d(pp,d)
   * \brief tests if point pp is on line d.
   * \ingroup structureintersection
   */
#define pointOnLine3d(pp,d) ( \
  ((x((d).v)*(y(pp)-y((d).p))) == (y((d).v)*(x(pp)-x((d).p)))) && \
  ((x((d).v)*(z(pp)-z((d).p))) == (z((d).v)*(x(pp)-x((d).p)))) \
)
  /** \def initializePolygon3d(pol)
   * \brief initialize acceleration datas for polygon intersection.
   * \ingroup structureintersection
   */
#define initializePolygon3d(pol) { \
 sVector3d v01; sousVectorPoint3d(v01,vertex(pol,0),vertex(pol,1)); normalizeVector3d(v01); \
 sVector3d v0n; sousVectorPoint3d(v0n,vertex(pol,0),vertex(pol,(pol).size-1)); normalizeVector3d(v0n); \
 sVector3d N; crossProductVector3d(N,v01,v0n); normalizeVector3d(N); \
 initPlane3d((pol).plane,vertex(pol,0),N); \
 sGeometricData Ai = ABS(x((pol).plane.v)); \
 sGeometricData Bi = ABS(y((pol).plane.v)); \
 sGeometricData Ci = ABS(z((pol).plane.v)); \
 (pol).coord = (Ai > Bi ? \
	      (Ai > Ci ? 0 : 2) : \
	      (Bi > Ci ? 1 : 2)); \
 for (int i=0;i<(pol).size;i++) { \
   int j = (i == ((pol).size)-1 ? 0 : i+1); \
   int k = (i == 0 ? ((pol).size)-1 : i-1); \
   point2d(vi,getu(vertex(pol,i),(pol).coord),getv(vertex(pol,i),(pol).coord)); \
   point2d(vj,getu(vertex(pol,j),(pol).coord),getv(vertex(pol,j),(pol).coord)); \
   point2d(vk,getu(vertex(pol,k),(pol).coord),getv(vertex(pol,k),(pol).coord)); \
   sVector2d vji; sousVectorPoint2d(vji,vi,vj); \
   line2d(dji,vi,vji); \
   sGeometricData ai = a(dji); \
   sGeometricData bi = b(dji); \
   sGeometricData ci = c(dji); \
   if ((ai*x(vk)+bi*y(vk)+ci) < epsilonValue) { initPoint2d((pol).aibi[i],ai,bi); (pol).ci[i] = ci; } \
   else { initPoint2d((pol).aibi[i],-ai,-bi); (pol).ci[i] = -ci; } \
 } \
}
  /** \def intersectLineLine2d(pp,d0,d1)
   * \brief compute intersection point pp between two 2d lines d0 and d1.
   * \ingroup structureintersection
   * assumes that d0 and d1 aren't parallels.
   */
#define intersectLineLine2d(pp,d0,d1) { \
  initPoint2d(pp , \
    (ABS(a(d0)) < smallEpsilonValue ? (((b(d1)*c(d0))-(c(d1)*b(d0)))/(a(d1)*b(d0))) : \
      (ABS(b(d0)) < smallEpsilonValue ? (-c(d0)/a(d0)) : \
      (ABS(a(d1)) < smallEpsilonValue ? (((b(d0)*c(d1))-(c(d0)*b(d1)))/(b(d1)*a(d0))) : \
      (ABS(b(d1)) < smallEpsilonValue ? (-c(d1)/a(d1)) : \
      ((B(d0)-B(d1))/(A(d1)-A(d0))))))) , \
    (ABS(a(d0)) < smallEpsilonValue ? (-c(d0)/b(d0)) : \
      (ABS(b(d0)) < smallEpsilonValue ? (((a(d1)*c(d0))-(c(d1)*a(d0)))/(b(d1)*a(d0))) : \
      (ABS(a(d1)) < smallEpsilonValue ? (-c(d1)/b(d1)) : \
      (ABS(b(d1)) < smallEpsilonValue ? (((a(d0)*c(d1))-(c(d0)*a(d1)))/(a(d1)*b(d0))) : \
      (A(d0)*((B(d0)-B(d1))/(A(d1)-A(d0)))+B(d0)))))) \
  ); \
}
  /** \def intersectLineLine3d(pp,d0,d1)
   * \brief compute intersection point pp between two 3d lines d0 and d1.
   * \ingroup structureintersection
   * assumes that d0 and d1 aren't parallels, but belongs to the same plane.
   */
#define intersectLineLine3d(pp,d0,d1) { \
 vector3d(vti,0.,0.,0.); \
 vector3d(vtj,0.,0.,0.); \
 crossProductVector3d(vti,(d0).v,(d1).v); \
 sousVectorPoint3d(vtj,(d0).p,(d1).p); \
 sGeometricData ti; ti = det3(vtj,(d1).v,vti) / sqr(normVector3d(vti)); \
 copyPoint3d((d0).v,pp); \
 mulVectorScalar3d(pp,ti); \
 addVectorPoint3d(pp,pp,(d0).p); \
}
  /** \def intersectLinePlane3d(pp,p0,d0)
   * \brief compute 3d intersection point pp between a plane p0 and a line d0.
   * \ingroup structureintersection
   * assumes that pp exists.
   */
#define intersectLinePlane3d(pp,p0,d0) { \
 sGeometricData ti = parameterPointOnLineIntersectingPlane3d(p0,d0); \
 copyPoint3d((d0).v,pp); \
 mulVectorScalar3d(pp,ti); \
 addVectorPoint3d(pp,pp,(d0).p); \
}
  /** \def intersectLinePolygon3d(pilp,pilp0,dilp0)
   * \brief compute 3d intersection point pilp between a polygon pilp0 and a line dilp0.
   * \ingroup structureintersection
   * assumes that pilp exist, but it can be outside polygon pilp0.
   */
#define intersectLinePolygon3d(pilp,pilp0,dilp0) { \
 int res = 1; \
 sPoint3d pinter; intersectLinePlane3d(pinter,(pilp0).plane,dilp0); \
 sGeometricData ui = getu(pinter,(pilp0).coord); \
 sGeometricData vi = getv(pinter,(pilp0).coord); \
 int i = 0; \
 while (i < (pilp0).size) { \
   if (((pilp0).aibi[i].x*ui+(pilp0).aibi[i].y*vi+(pilp0).ci[i]) >= epsilonValue) { res = 0; break; } \
   i++; \
 } \
 if (res) { copyPoint3d(pinter,pilp); } \
 else { copyPoint3d((dilp0).p,pilp); } \
}

#ifdef USENAMESPACE
} /* namespace Geometric */
#endif /* #ifdef USENAMESPACE */

#endif /* #ifndef _GEOMETRIC_INTERSECTFUNCTIONS_H_ */
