#ifndef _MATH_NRCHAPTER02SECTION03_H_
#define _MATH_NRCHAPTER02SECTION03_H_

/*!
 * \file Math/NRChapter02Section03.h
 * \brief mathematical functions on float vetor/matrix.
 *
 * mathematical algorithms described in Numerical Recipes Chapter 2, Section 3.
 */

#include "Math/includes.h"

/** \defgroup chapitre02section03 2.3 LU Decomposition and its Applications
 * \ingroup chapitre02
 */

#ifdef USENAMESPACE
namespace NumericalRecipes
{
#endif /* #ifdef USENAMESPACE */

  /*!
    \fn void ludcmp(double **a,int n,int *indx,double *d)
    \brief LU decomposition of a matrix
    \ingroup chapitre02section03
    \param a is the input matrix.a is replaced by the LU decomposition.
    \param n is the size of a.
    \param indx returns the vector recording the row permutation.
    \param d returns -1 or +1.

    As described in "Numerical Recipes in c" pp. 46-47

    Given a matrix a[1..n][1..n], this routine replaces it by the LU decomposition of a rowwise permutation of itself. a and n are input. a is output, arranged as in equation 2.3.14 (see NR); indx[1..n] is an output vector that records the row permutation effected by the partial pivoting; d is output as +/-1 depending on whether the number of row interchanges was even or odd, respectively. This routine is used in combination with lubksb to solve linear equations or invert a matrix.
  */
  extern void ludcmp(double **a,int n,int *indx,double *d);
  /*!
    \fn void lubksb(double **a,int n,int *indx,double b[])
    \brief LU backsubstitution
    \ingroup chapitre02section03
    \param a is the LU decomposition of A, determined by ludcmp.
    \param n defines the size of a.
    \param indx is the permutation vector returned by ludcmp.
    \param b is the input right-hand side vector. b is replaced by the solution vector x.
    \sa ludcmp()

    As described in "Numerical Recipes in c" pp. 47-48

    Solves the set of n linear equations A.X=B. Here a[1..n][1..n] is input, not as the matrix A but rather as its LU decomposition, determined by the routine ludcmp. indx[1..n] is input as the permutation vector returned by ludcmp. b[1..n] is input as the right-hand side vector B, and returns with the solution vector X. a, n, and indx are not modified by this routine and can be left in place for successive calls with different right-hand sides b. This routine takes into account the possibility that b begin with many zero elements, so it's efficient for use in matrix inversion.
  */
  extern void lubksb(double **a,int n,int *indx,double b[]);

#ifdef USENAMESPACE
} /* namespace NumericalRecipes */
#endif /* #ifdef USENAMESPACE */

#endif /* #ifndef _MATH_NRCHAPTER02SECTION03_H_ */
