#ifndef _MATH_NRCHAPTER02SECTION04_H_
#define _MATH_NRCHAPTER02SECTION04_H_

/*!
 * \file Math/NRChapter02Section04.h
 * \brief mathematical functions on float vetor/matrix.
 *
 * mathematical algorithms described in Numerical Recipes Chapter 2, Section 4.
 */

#include "Math/includes.h"

/** \defgroup chapitre02section04 2.4 Tridiagonal and Band Diagonal Systems of Equations
 * \ingroup chapitre02
 */

#ifdef USENAMESPACE
namespace NumericalRecipes
{
#endif /* #ifdef USENAMESPACE */

  /*!
    \fn void tridag(double a[],double b[],double c[],double r[],double u[],long n)
    \brief resolution of a tridiagonal system of equations.
    \ingroup chapitre02section04
    \param a first group of the coefficients.
    \param b second group of the coefficients.
    \param c third group of the coefficients.
    \param r values of the equations.
    \param u returns the values of the unknowns.
    \param n size of the vectors.

    As described in "Numerical Recipes in c" pp. 51

    Solves for a vector u[1..n] the tridiagonal linear set given by equation 2.4.1 (see NR). a[1..n], b[1..n], c[1..n], and r[1..n] are input vectors and are not modified.
  */
  extern void tridag(double a[],double b[],double c[],double r[],double u[],long n);
  /*!
    \fn void banmul(double** a,long n,int m1,int m2,double x[],double b[])
    \brief band-diagonal matrix multiplication.
    \ingroup chapitre02section04
    \param a the coefficients of the equations.
    \param n number of equations/unknowns.
    \param m1 index of diagonal elements in matrix a (see below).
    \param m2 index of superdiagonal elements of matrix a (see below).
    \param x the vector of unknowns.
    \param b the vector containing the values of the equations.

    As described in "Numerical Recipes in c" pp. 52

    Matrix multiply b = A . x, where A is band diagonal with m1 rows below the diagonal and m2 rows above. The input vector x and output vector b are stored as x[1..n] and b[1..n], respectively. The array a[1..n][1..m1+m2+1] stores A as follows: The diagonal elements are in a[1..n][m1+1]. Subdiagonal elements are in a[j..n][1..m1] (with j > 1 appropriate to the number of elements on each subdiagonal). Superdiagonal elements are in a[1..j][m1+2..m1+m2+1] with j < n appropriate to the number of elements on each superdiagonal.
   */
  extern void banmul(double** a,long n,int m1,int m2,double x[],double b[]);
  /*!
    \fn void bandec(double **a,long n, int m1, int m2, double **al,long indx[], double *d)
    \brief bandec is the band-diagonal analog of ludcmp.
    \ingroup chapitre02section04
    \param a the coefficients of the equations.
    \param n number of equations/unknowns.
    \param m1 index of diagonal elements in matrix a (see below).
    \param m2 index of superdiagonal elements of matrix a (see below).
    \param al returns the lower triangular matrix.
    \param indx returns the row permutation vector.
    \param d tells if the number of row permutation is odd or even.

    As described in "Numerical Recipes in c" pp. 53

    Given an n x n band diagonal matrix A with m1 subdiagonal rows and m2 superdiagonal rows, compactly stored in the array a[1..n][1..m1+m2+1] as described in the comment for routine banmul, this routine constructs an LU decomposition of a rowwise permutation of A. The upper triangular matrix replaces a, while the lower triangular matrix is returned in al[1..n][1..m1]. indx[1..n] is an output vector which records the row permutation e ected by the partial pivoting; d is output as  1 depending on whether the number of row interchanges was even or odd, respectively. This routine is used in combination with banbks to solve band-diagonal sets of equations.
  */
  extern void bandec(double **a,long n, int m1, int m2, double **al,long indx[], double *d);
  /*!
    \fn void banbks(double **a, long n, int m1, int m2, double **al,long indx[], double b[])
    \brief band diagonal linear equations solving.
    \ingroup chapitre02section04
    \param a the coefficients of the equations.
    \param n number of equations/unknowns.
    \param m1 index of diagonal elements in matrix a (see below).
    \param m2 index of superdiagonal elements of matrix a (see below).
    \param al the lower triangular matrix.
    \param indx the row permutation vector.
    \param b the right-hand side vector.

    As described in "Numerical Recipes in c" pp. 54

    Given the arrays a, al, and indx as returned from bandec, and given a right-hand side vector b[1..n], solves the band diagonal linear equations A .  x = b. The solution vector x overwrites b[1..n]. The other input arrays are not modified, and can be left in place for successive calls with different right-hand sides.
  */
  extern void banbks(double **a, unsigned long n, int m1, int m2, double **al,long indx[], double b[]);

#ifdef USENAMESPACE
} /* namespace NumericalRecipes */
#endif /* #ifdef USENAMESPACE */

#endif /* #ifndef _MATH_NRCHAPTER02SECTION04_H_ */
