#ifndef _MATH_NRCHAPTER02SECTION06_H_
#define _MATH_NRCHAPTER02SECTION06_H_

/*!
 * \file Math/NRChapter02Section06.h
 * \brief mathematical functions on float vetor/matrix.
 *
 * mathematical algorithms described in Numerical Recipes Chapter 2, Section 6.
 */

#include "Math/includes.h"

/** \defgroup chapitre02section06 2.6 Singular Value Decomposition
 * \ingroup chapitre02
 */

#ifdef USENAMESPACE
namespace NumericalRecipes
{
#endif /* #ifdef USENAMESPACE */

  /*!
    \fn void svbksb(double **u,double w[],double **v,int m,int n,double b[],double x[])
    \brief SVD backsubstitution
    \ingroup chapitre02section06
    \param u is the matrix U from SVD decomposition of A.
    \param w is the vector representing the diagonal matrix of singular values W.
    \param v is the matrix V (not V') from the SVD decomposition process.
    \param m is the number of equations.
    \param n is the number of unknowns.
    \param b is the input right-hand side vector.
    \param x returns the solution vector.
    \sa svdcmp()

    As described in "Numerical Recipes in c" p. 64.

    Solves A.X=B for a vector X, where A is specified by the arrays u[1..m][1..n], w[1..n],v[1..n][1..n] as returned by svdcmp. m and n are the dimensions of a, and will be equal for square matrices. b[1..m] is the input right-hand side. x[1..n] is the output solution vector. No input quantities are destroyed, so the routine may be called sequentially with different d\'s.
  */
  extern void svbksb(double **u,double w[],double **v,int m,int n,double b[],double x[]);
  /*!
    \fn void svdcmp(double **a,int m,int n,double w[],double **v)
    \brief SVD decomposition of a matrix
    \ingroup chapitre02section06
    \param a  is the input matrix. a is replaced by the matrix U.
    \param m is the number of rows of a.
    \param n is the number of columns of a.
    \param w returns the vector describing W.
    \param v returns the matrix V.
    \sa pythag()

    As described in "Numerical Recipes in c" pp. 67-70.

    Given a matrix a[1..m][1..n], this routine computes its singular value decomposition, A= U.W.V'. The matrix U replaces a on output. The diagonal matrix of singular values W is output as a vector w[1..n]. The matrix V (not the transpose V') is output as v[1..n][1..n].
  */
  extern void svdcmp(double **a,int m,int n,double w[],double **v);
  /*!
    \fn double pythag(double a,double b)
    \brief computes the squares distance between two doubles
    \ingroup chapitre02section06
    \param a the first double value
    \param b the second double value
    \return (a^2+b^2)^(1/2)

    As described in "Numerical Recipes in c" pp. 70

    Computes (a^2+b^2)^(1/2) without destructive underflow or overflow.
  */
  extern double pythag(double a,double b);

#ifdef USENAMESPACE
} /* namespace NumericalRecipes */
#endif /* #ifdef USENAMESPACE */

#endif /* #ifndef _MATH_NRCHAPTER02SECTION06_H_ */
