#ifndef _MATH_NRCHAPTER02SECTION07_H_
#define _MATH_NRCHAPTER02SECTION07_H_

/*!
 * \file Math/NRChapter02Section07.h
 * \brief mathematical functions on float vetor/matrix.
 *
 * mathematical algorithms described in Numerical Recipes Chapter 2, Section 7.
 */

#include "Math/includes.h"

/** \defgroup chapitre02section07 2.7 Sparse Linear Systems
 * \ingroup chapitre02
 */

#ifdef USENAMESPACE
namespace NumericalRecipes
{
#endif /* #ifdef USENAMESPACE */

  /*!
    \fn void cyclic(double a[], double b[], double c[], double alpha, double beta,double r[], double x[], long n)
    \brief cyclic linear equations solving.
    \ingroup chapitre02section07
    \param a first group of the coefficients.
    \param b second group of the coefficients.
    \param c third group of the coefficients.
    \param alpha upper corner entry in the matrix.
    \param beta lower corner entry in the matrix.
    \param r the right-hand side of the equations.
    \param x returns the values of the unknowns.
    \param n size of the vectors.

    As described in "Numerical Recipes in c" pp. 75

    Solves for a vector x[1..n] the "cyclic" set of linear equations given by equation (2.7.9). a, b, c, and r are input vectors, all dimensioned as [1..n], while alpha and beta are the corner entries in the matrix. The input is not modified.
  */
  extern void cyclic(double a[], double b[], double c[], double alpha, double beta,double r[], double x[], long n);
  /*!
    \fn void sprsin(double **a, int n, double thresh, unsigned long nmax, double sa[], unsigned long ija[])
    \brief full storage to row-indexed sparse storage matrix conversion.
    \ingroup chapitre02section07
    \param a the input matrix.
    \param n size of the matrix.
    \param thresh threshold on matrix values
    \param nmax maximum size of output vectors.
    \param sa output vector containing elements of matrix a.
    \param ija output vector containing index on elements of sa.

    As described in "Numerical Recipes in c" pp. 79

    Converts a square matrix a[1..n][1..n] into row-indexed sparse storage mode. Only elements of a with magnitude  thresh are retained. Output is in two linear arrays with dimension nmax (an input parameter): sa[1..] contains array values, indexed by ija[1..]. The number of elements filled of sa and ija on output are both ija[ija[1]-1]-1 (see text).
  */
  extern void sprsin(double **a, int n, double thresh, unsigned long nmax, double sa[], unsigned long ija[]);
  /*!
    \fn void sprsax(double sa[], unsigned long ija[], double x[], double b[],unsigned long n)
    \brief row-indexed sparse storage matrix multiplication.
    \ingroup chapitre02section07
    \param sa input vector containing elements of matrix a.
    \param ija input vector containing index on elements of sa.
    \param x input vector, used to multiply the matrix.
    \param b output vector (result of the operation).
    \param n size of the vectors.

    As described in "Numerical Recipes in c" pp. 79-80

    Multiply a matrix in row-index sparse storage arrays sa and ija by a vector x[1..n], giving a vector b[1..n].
  */
  extern void sprsax(double sa[], unsigned long ija[], double x[], double b[],unsigned long n);
  /*!
    \fn void sprstx(double sa[], unsigned long ija[], double x[], double b[],unsigned long n)
    \brief row-indexed sparse storage transpose matrix multiplication.
    \ingroup chapitre02section07
    \param sa input vector containing elements of matrix a.
    \param ija input vector containing index on elements of sa.
    \param x input vector, used to multiply the matrix.
    \param b output vector (result of the operation).
    \param n size of the vectors.

    As described in "Numerical Recipes in c" pp. 80

    Multiply the transpose of a matrix in row-index sparse storage arrays sa and ija by a vector x[1..n], giving a vector b[1..n].
  */
  extern void sprstx(double sa[], unsigned long ija[], double x[], double b[],unsigned long n);
  /*!
    \fn void sprstp(double sa[], unsigned long ija[], double sb[], unsigned long ijb[])
    \brief row-indexed sparse storage transpose matrix construction.
    \ingroup chapitre02section07
    \param sa input vector containing elements of matrix a.
    \param ija input vector containing index on elements of sa.
    \param sb output vector containing elements of transpose of matrix a.
    \param ijb output vector containing index on elements of sb.

    As described in "Numerical Recipes in c" pp. 80-81

    Construct the transpose of a sparse square matrix, from row-index sparse storage arrays sa and ija into arrays sb and ijb.
  */
  extern void sprstp(double sa[], unsigned long ija[], double sb[], unsigned long ijb[]);
  /*!
    \fn void sprspm(double sa[], unsigned long ija[], double sb[], unsigned long ijb[],double sc[], unsigned long ijc[])
    \brief row-indexed sparse storage matrices multiplication.
    \ingroup chapitre02section07
    \param sa input vector containing elements of matrix a.
    \param ija input vector containing index on elements of sa.
    \param sb input vector containing elements of matrix b.
    \param ijb input vector containing index on elements of sb.
    \param sc output vector containing elements of the resulting matrix.
    \param ijc output vector containing index on elements of sc.

    As described in "Numerical Recipes in c" pp. 82

    Matrix multiply A . BT where A and B are two sparse matrices in row-index storage mode, and BT is the transpose of B. Here, sa and ija store the matrix A; sb and ijb store the matrix B. This routine computes only those components of the matrix product that are pre-specified by the input index array ijc, which is not modi ed. On output, the arrays sc and ijc give the product matrix in row-index storage mode. For sparse matrix multiplication, this routine will often be preceded by a call to sprstp, so as to construct the transpose of a known matrix into sb, ijb.
  */
  extern void sprspm(double sa[], unsigned long ija[], double sb[], unsigned long ijb[],double sc[], unsigned long ijc[]);
  /*!
    \fn void sprstm(double sa[], unsigned long ija[], double sb[], unsigned long ijb[], double thresh, unsigned long nmax, double sc[], unsigned long ijc[])
    \brief row-indexed sparse storage matrices multiplication.
    \ingroup chapitre02section07
    \param sa input vector containing elements of matrix a.
    \param ija input vector containing index on elements of sa.
    \param sb input vector containing elements of matrix b.
    \param ijb input vector containing index on elements of sb.
    \param thresh threshold on resulting matrix values
    \param nmax maximum size of output matrix.
    \param sc output vector containing elements of the resulting matrix.
    \param ijc output vector containing index on elements of sc.

    As described in "Numerical Recipes in c" pp. 82-83

    Matrix multiply A   BT where A and B are two sparse matrices in row-index storage mode, and BT is the transpose of B. Here, sa and ija store the matrix A; sb and ijb store the matrix B. This routine computes all components of the matrix product (which may be non-sparse!), but stores only those whose magnitude exceeds thresh. On output, the arrays sc and ijc (whose maximum size is input as nmax) give the product matrix in row-index storage mode. For sparse matrix multiplication, this routine will often be preceded by a call to sprstp, so as to construct the transpose of a known matrix into sb, ijb.
  */
  extern void sprstm(double sa[], unsigned long ija[], double sb[], unsigned long ijb[], double thresh, unsigned long nmax, double sc[], unsigned long ijc[]);
  /*!
    \fn void linbcg(unsigned long n, double b[], double x[], int itol, double tol,int itmax, int *iter, double *err)
    \brief linear equations solving by iterative biconjugate gradient method.
    \ingroup chapitre02section07
    \param n size of the vectors.
    \param b the right-hand side of the equations.
    \param x the initial guess, replaced by the final solution.
    \param itol specify the convergence test to use.
    \param tol specify the desired convergence tolerance.
    \param itmax the maximum number of iteration to do.
    \param iter returns the actual number of iterations done.
    \param err returns the estimated error.
    \sa asolve(), atimes(), snrm()

    As described in "Numerical Recipes in c" pp. 86-88

    Solves A  x = b for x[1..n], given b[1..n], by the iterative biconjugate gradient method. On input x[1..n] should be set to an initial guess of the solution (or all zeros); itol is 1,2,3, or 4, specifying which convergence test is applied (see text); itmax is the maximum number of allowed iterations; and tol is the desired convergence tolerance. On output, x[1..n] is reset to the improved solution, iter is the number of iterations actually taken, and err is the estimated error. The matrix A is referenced only through the user-supplied routines atimes, which computes the product of either A or its transpose on a vector; and asolve, which solves eAx = b or eATx = b for some preconditioner matrix eA (possibly the trivial diagonal part of A).
  */
  extern void linbcg(unsigned long n, double b[], double x[], int itol, double tol,int itmax, int *iter, double *err);
  /*!
    \fn double snrm(unsigned long n, double sx[], int itol)
    \brief used by linbcg for vector norm computation.
    \ingroup chapitre02section07
    \param n size of the vector.
    \param sx the vector.
    \param itol the norm to compute.
    \return the norm of sx.
    \sa linbcg()

    As described in "Numerical Recipes in c" pp. 88

    Compute one of two norms for a vector sx[1..n], as signaled by itol.
  */
  extern double snrm(unsigned long n, double sx[], int itol);
  /*!
    \fn void atimes(unsigned long n, double x[], double r[], int itrnsp)
    \brief used by linbcg for sparse multiplication.
    \ingroup chapitre02section07
    \param n size of the vectors.
    \param x input vector.
    \param r output vector.
    \param itrnsp the kind of multiplication (matrix or transpose).
    \sa sprsax(), sprstx()

    As described in "Numerical Recipes in c" pp. 88-89
  */
  extern void atimes(unsigned long n, double x[], double r[], int itrnsp);
  /*!
    \fn void asolve(unsigned long n, double b[], double x[], int itrnsp)
    \brief used by linbcg for preconditionner.
    \ingroup chapitre02section07
    \param n size of the vectors.
    \param b input vector.
    \param x output vector.
    \param itrnsp not used.

    As described in "Numerical Recipes in c" pp. 89
  */
  extern void asolve(unsigned long n, double b[], double x[], int itrnsp);

#ifdef USENAMESPACE
} /* namespace NumericalRecipes */
#endif /* #ifdef USENAMESPACE */

#endif /* #ifndef _MATH_NRCHAPTER02SECTION07_H_ */
