#ifndef _MATH_NRCHAPTER02SECTION10_H_
#define _MATH_NRCHAPTER02SECTION10_H_

/*!
 * \file Math/NRChapter02Section10.h
 * \brief mathematical functions on float vetor/matrix.
 *
 * mathematical algorithms described in Numerical Recipes Chapter 2, Section 10.
 */

#include "Math/includes.h"

/** \defgroup chapitre02section10 2.10 QR Decomposition
 * \ingroup chapitre02
 */

#ifdef USENAMESPACE
namespace NumericalRecipes
{
#endif /* #ifdef USENAMESPACE */

  /*!
    \fn void qrdcmp(double **a, int n, double *c, double *d, int *sing)
    \brief QR decomposition
    \ingroup chapitre02section10
    \param a the input matrix. Returns the R matrix.
    \param n size of the matrix.
    \param c returns some coefficient for the orthogonal matrix Q.
    \param d returns diagonal elements of the decomposition.
    \param sing specify if a singularity has been encountered.

    As described in "Numerical Recipes in c" pp. 99

    Constructs the QR decomposition of a[1..n][1..n]. The upper triangular matrix R is returned in the upper triangle of a, except for the diagonal elements of R which are returned in d[1..n]. The orthogonal matrix Q is represented as a product of n1 Householder matrices Q1 : : :Qn1, where Qj =1uj  uj=cj. The ith component of uj is zero for i = 1; : : : ; j1 while the nonzero components are returned in a[i][j] for i = j; : : :; n. sing returns as true (1) if singularity is encountered during the decomposition, but the decomposition is still completed in this case; otherwise it returns false (0).
  */
  extern void qrdcmp(double **a, int n, double *c, double *d, int *sing);
  /*!
    \fn void qrsolv(double **a, int n, double c[], double d[], double b[])
    \brief QR backsubstitution
    \ingroup chapitre02section10
    \param a the input matrix. contains the R matrix.
    \param n size of the matrix.
    \param c some coefficient for the orthogonal matrix Q.
    \param d diagonal elements of the decomposition.
    \param b the right-hand side vector. returns the solution vector.

    As described in "Numerical Recipes in c" pp. 100

    Solves the set of n linear equations A  x = b. a[1..n][1..n], c[1..n], and d[1..n] are input as the output of the routine qrdcmp and are not modified. b[1..n] is input as the right-hand side vector, and is overwritten with the solution vector on output.
  */
  extern void qrsolv(double **a, int n, double c[], double d[], double b[]);
  /*!
    \fn void rsolv(double **a, int n, double d[], double b[])
    \brief right triangular backsubstitution
    \ingroup chapitre02section10
    \param a the input matrix. contains the R matrix.
    \param n size of the matrix.
    \param d diagonal elements of the decomposition.
    \param b the right-hand side vector. returns the solution vector.

    As described in "Numerical Recipes in c" pp. 100

    Solves the set of n linear equations R  x = b, where R is an upper triangular matrix stored in a and d. a[1..n][1..n] and d[1..n] are input as the output of the routine qrdcmp and are not modied. b[1..n] is input as the right-hand side vector, and is overwritten with the solution vector on output.
  */
  extern void rsolv(double **a, int n, double d[], double b[]);
  /*!
    \fn void qrupdt(double **r, double **qt, int n, double u[], double v[])
    \brief update a QR decomposition
    \ingroup chapitre02section10
    \param r the R matrix.
    \param qt the Q^t matrix.
    \param n size of the matrices.
    \param u some coefficients.
    \param v some coefficients.

    As described in "Numerical Recipes in c" pp. 101

    Given the QR decomposition of some n x n matrix, calculates the QR decomposition of the matrix Q(R+u v). The quantities are dimensioned as r[1..n][1..n], qt[1..n][1..n], u[1..n], and v[1..n]. Note that QT is input and returned in qt.
  */
  extern void qrupdt(double **r, double **qt, int n, double u[], double v[]);
  /*!
    \fn void rotate(double **r, double **qt, int n, int i, double a, double b)
    \brief Jacobi rotation by qrupdt
    \ingroup chapitre02section10
    \param r the R matrix.
    \param qt the Q^t matrix.
    \param n size of the matrices.
    \param i index on the rows.
    \param a first parameter of the rotation.
    \param b second parameter of the rotation.

    As described in "Numerical Recipes in c" pp. 101

    Given matrices r[1..n][1..n] and qt[1..n][1..n], carry out a Jacobi rotation on rows i and i + 1 of each matrix. a and b are the parameters of the rotation: cos  = a=pa2 + b2, sin  = b=pa2 + b2.
  */
  extern void rotate(double **r, double **qt, int n, int i, double a, double b);

#ifdef USENAMESPACE
} /* namespace NumericalRecipes */
#endif /* #ifdef USENAMESPACE */

#endif /* #ifndef _MATH_NRCHAPTER02SECTION10_H_ */
