#ifndef _MATH_NRCHAPTER06SECTION02_H_
#define _MATH_NRCHAPTER06SECTION02_H_

/*!
 * \file Math/NRChapter06Section02.h
 * \brief mathematical functions on float vetor/matrix.
 *
 * mathematical algorithms described in Numerical Recipes Chapter 6, Section 2.
 */

#include "Math/includes.h"

/** \defgroup chapitre06section02 6.2 Incomplete Gamma Function, Error Function, Chi-Square Probability Function, Cumulative Poisson Function
 * \ingroup chapitre06
 */

#ifdef USENAMESPACE
namespace NumericalRecipes
{
#endif /* #ifdef USENAMESPACE */

  /*!
    \fn double gammq(double a,double x)
    \brief complement of incomplete gamma function
    \ingroup chapitre06section02
    \param x supposed to be >= 0.
    \param a supposed to be > 0.
    \return the value of the incomplete gamma function Q(a,x) .
    \sa gcf(), gser()

    As described in "Numerical Recipes in c" p. 218.

    Returns the incomplete gamma function at point (a,x) : Q(a,x) = 1. - P(a,x).
  */
  extern double gammq(double a,double x);
  /*!
    \fn void gser(double *gamser,double a,double x,double *gln)
    \brief series used by gammp and gammq
    \ingroup chapitre06section02
    \param gamser = P(a,x), returns the value of the incomplete gamma function.
    \param x supposed to be >= 0.
    \param a supposed to be > 0.
    \param gln returns the gamma function at point a.
    \sa gammln()

    As described in "Numerical Recipes in c" pp. 218-219.

    Returns the incomplete gamma function P(a,x) evaluated by its series representation as gamser. Also returns ln(gamma func(a)) as gln.
  */
  extern void gser(double *gamser,double a,double x,double *gln);
  /*!
    \fn void gcf(double *gammcf,double a,double x,double *gln)
    \brief continued fraction used by gammp and gammq
    \ingroup chapitre06section02
    \param x supposed to be >= 0.
    \param a supposed to be > 0.
    \param gammcf = Q(a,x), returns the value of the incomplete gamma function.
    \param gln returns the neperian logarithm of the gamma function at point a.
    \sa gammln()

    As described in "Numerical Recipes in c" p. 219.

    Returns the incomplete gamma function Q(a,x) evaluated by its continued fraction representation as gammcf. Also returns ln(gamma func(a)) as gln.
  */
  extern void gcf(double *gammcf,double a,double x,double *gln);

#ifdef USENAMESPACE
} /* namespace NumericalRecipes */
#endif /* #ifdef USENAMESPACE */

#endif /* #ifndef _MATH_NRCHAPTER06SECTION02_H_ */
