#ifndef _MATH_DMATH_H_
#define _MATH_DMATH_H_

/*!
  \file Math/dmath.h
  \brief mathematical functions on double values

  Double utility routines used by all, or some, of the mathematical algorithms.
 */

#include "Math/includes.h"

#ifdef USENAMESPACE
namespace NumericalRecipes
{
#endif /* #ifdef USENAMESPACE */

  //@{
  /** \name 1d/2d float array memory manipulation. */
  /*!
    \fn double\\* dvector(long nl, long nh)
    \brief dynamically creates an 1D array of doubles
    \ingroup chapitre1
    \param nl index on the first element
    \param nh index on the last element
    \return pointer on the array
  */
  double * dvector(long nl, long nh);
  /*!
    \fn double\\*\\* dmatrix(long nrl, long nrh, long ncl, long nch)
    \brief dynamically creates a 2D array of doubles
    \ingroup chapitre1
    \param nrl index on the first row element
    \param nrh index on the last row element
    \param ncl index on the first column element
    \param nch index on the last column element
    \return pointer on the array
  */
  double **dmatrix(long nrl, long nrh, long ncl, long nch);
  /*!
    \fn void free_vector(double *v, long nl, long nh)
    \brief deletes an 1D array of doubles dynamically created with dvector
    \ingroup chapitre1
    \param v pointer on the array
    \param nl index on the first element
    \param nh index on the last element
    \sa dvector
  */
  void free_vector(double *v, long nl, long nh);
  /*!
    \fn void free_matrix(double **m, long nrl, long nrh, long ncl, long nch)
    \brief deletes a 2D array of doubles dynamically created with dmatrix
    \ingroup chapitre1
    \param m pointer on the array
    \param nrl index on the first row element
    \param nrh index on the last row element
    \param ncl index on the first column element
    \param nch index on the last column element
    \sa dmatrix
  */
  void free_matrix(double **m, long nrl, long nrh, long ncl, long nch);
  //@}
  //@{
  /** \name 1d/2d float array manipulation functions. */
  /*!
    \fn void print_vector(double * b,long nl,long nh)
    \brief prints an 1D array of doubles dynamically created with dvector
    \ingroup chapitre16
    \param b pointer on the array
    \param nl index on the first element
    \param nh index on the last element
  */
  void print_vector(double * b,long nl,long nh);
  /*!
    \fn void print_matrix(double **b,long nrl,long nrh,long ncl,long nch)
    \brief prints a 2D array of doubles dynamically created with dmatrix
    \ingroup chapitre16
    \param b pointer on the array
    \param nrl index on the first row element
    \param nrh index on the last row element
    \param ncl index on the first column element
    \param nch index on the last column element
  */
  void print_matrix(double **b,long nrl,long nrh,long ncl,long nch);
  /*!
    \fn void mult(double **mn,double *p,double *d,long n,long m)
    \brief multiply two 1d vectors.
    \ingroup chapitre16
    \param mn double**, pointer on the resulting matrix
    \param p double*, pointer on the first vector
    \param d double*, pointer on the second vector
    \param n long, length of vector p
    \param m long, length of vector d
  */
  void mult(double **mn,double *p,double *d,long n,long m);
  /*!
    \fn void mult_matrix(double **s1,double **s2,double **d,long nr,long nc,long ns)
    \brief multiply two 2d matrices.
    \ingroup chapitre16
    \param s1 int**, pointer on the first matrix
    \param s2 int**, pointer on the second matrix
    \param d int**, pointer on the resulting matrix
    \param nr long, number of lines in matrix s1
    \param nc long, number of columns in matrix s2
    \param ns long, number of columns (resp. lines) in matrix s1 (resp. s2).
  */
  void mult_matrix(double **s1,double **s2,double **d,long nr,long nc,long ns);
  /*!
    \fn void copy_vector(double *s,double *d,long nl,long nh)
    \brief copy of a 1d vector.
    \ingroup chapitre16
    \param s double*, pointer on the source vector
    \param d double*, pointer on the destination vector
    \param nl long, index on the first element
    \param nh long, index on the last element
  */
  void copy_vector(double *s,double *d,long nl,long nh);
  /*!
    \fn void copy_matrix(double **s,double **d,long nrl,long nrh,long ncl,long nch)
    \brief copy of a 2d matrix.
    \ingroup chapitre16
    \param s double**, pointer on the source matrix
    \param d double**, pointer on the destination matrix
    \param nrl long, index on the first row element
    \param nrh long, index on the last row element
    \param ncl long, index on the first column element
    \param nch long, index on the last column element
  */
  void copy_matrix(double **s,double **d,long nrl,long nrh,long ncl,long nch);
  //@}

#ifdef USENAMESPACE
} /* namespace NumericalRecipes */
#endif /* #ifdef USENAMESPACE */

#endif /* #ifdef _MATH_DMATH_H_ */
