#ifndef _MATH_FUNCTIONS_H_
#define _MATH_FUNCTIONS_H_

/*!
 * \file Math/functions.h
 * \brief mathematical functions on float vetor/matrix.
 *
 * Integer utility routines used by all, or some, of the mathematical algorithms.
 */

#include "Math/includes.h"

/** \defgroup others Other Functions
 */

#ifdef USENAMESPACE
/*! \namespace NumericalRecipes
 * Namespace containing mathematic algorithms from Numerical Recipes.
 */
namespace NumericalRecipes
{
#endif /* #ifdef USENAMESPACE */

  //@{
  /*!
    \fn double det(double **a,long n)
    \ingroup others
    \param a is the source matrix.
    \param n defines the size of s.
    \return the value of det(s).
    \see ludcmp(), lubksb()

    Utility routine.

    Given a source matrix s[1..n][1..n], it returns the determinant of s.
  */
  double det(double **a,long n);
  /** \name matrix manipulation functions */
  /*!
    \fn void transpose(double **s,double **d,long nrl,long nrh,long ncl,long nch)
    \brief matrix transposition
    \ingroup others
    \param s is the source matrix.
    \param d returns the transposition of s.
    \param nrl index on the first row element
    \param nrh index on the last row element
    \param ncl index on the first column element
    \param nch index on the last column element

    Utility routine.

    Given a source matrix s[nrl..nrh][ncl..nch], it returns the transposed matrix d[ncl..nch][nrl..nrh].
  */
  void transpose(double **s,double **d,long nrl,long nrh,long ncl,long nch);
  /*!
    \fn void inv(double **s,double **ds,long n)
    \brief matrix inversion
    \ingroup others
    \param s is the source matrix.
    \param ds returns the inverse matrix of s.
    \param n defines the size of s.
    \sa ludcmp() and lubksb()

    Utility routine.

    Given a source matrix s[1..n][1..n], it returns the inverse matrix d[1..n][1..n].
  */
  void inv(double **s,double **ds,long n);
  /*!
    \fn void pinv(double** s,double** d,long nr,long nc)
    \brief matrix pseudo-inversion
    \ingroup others
    \param s is the source matrix.
    \param d returns the inverse matrix of s.
    \param nr defines the number of rows of s.
    \param nc defines the number of columns of s.
    \sa transpose() and inv()

    Utility routine.

    Given a source matrix s[1..nr][1..nc], it returns the pseudo-inverse matrix d[1..nc][1..nr].
  */
  void pinv(double** s,double** d,long nr,long nc);
  //@}
  //@{
  /** \name equation system resolution */
  /*!
    \fn double* SolveLU(double** A,double* b,int card,int nx)
    \brief equation solver by svd decomposition.
    \ingroup others
    \param A the matrix containing equations coefficients.
    \param b the vector containing equations values.
    \param card the number of equations.
    \param nx the number of unknowns.
    \return the vector containing the unknowns.
  */
  double* SolveLU(double** A,double* b,int card,int nx);
  /*!
    \fn double* SolveSVD(double** A,double* b,int card,int nx)
    \brief equation solver by svd decomposition.
    \ingroup others
    \param A the matrix containing equations coefficients.
    \param b the vector containing equations values.
    \param card the number of equations.
    \param nx the number of unknowns.
    \return the vector containing the unknowns.
  */
  double* SolveSVD(double** A,double* b,int card,int nx);
  /*!
    \fn double* SolvePINV(double** A,double* b,int card,int nx)
    \brief equation solver by pseudo-inverse method.
    \ingroup others
    \param A the matrix containing equations coefficients.
    \param b the vector containing equations values.
    \param card the number of equations.
    \param nx the number of unknowns.
    \return the vector containing the unknowns.
  */
  double* SolvePINV(double** A,double* b,int card,int nx);
  //@}

#ifdef USENAMESPACE
} /* namespace NumericalRecipes */
#endif /* #ifdef USENAMESPACE */

#endif /* #ifndef _MATH_FUNCTIONS_H_ */
