#ifndef _MATH_IMATH_H_
#define _MATH_IMATH_H_

/*!
 * \file Math/imath.h
 * \brief mathematical functions on integer values.
 *
 * Integer utility routines used by all, or some, of the mathematical algorithms.
 */

#include "Math/includes.h"

#ifdef USENAMESPACE
namespace NumericalRecipes
{
#endif /* #ifdef USENAMESPACE */

  //@{
  /** \name 1d/2d long array memory manipulation. */
  /*!
    \fn long *lvector(long nl, long nh)
    \brief dynamically creates an 1D array of long
    \ingroup chapitre1
    \param nl long, index on the first element
    \param nh long, index on the last element
    \return pointer on the array
  */
  long *lvector(long nl, long nh);
  /*!
    \fn void free_vector(long *v, long nl, long nh)
    \brief deletes an 1D array of long dynamically created with lvector
    \ingroup chapitre1
    \param v long*, pointer on the array
    \param nl long, index on the first element
    \param nh long, index on the last element
    \sa lvector
  */
  void free_vector(long *v, long nl, long nh);
  //@}
  //@{
  /** \name 1d/2d integer array memory manipulation. */
  /*!
    \fn int *ivector(long nl, long nh)
    \brief dynamically creates an 1D array of integers
    \ingroup chapitre1
    \param nl long, index on the first element
    \param nh long, index on the last element
    \return pointer on the array
  */
  int *ivector(long nl, long nh);
  /*!
    \fn int **imatrix(long nrl, long nrh, long ncl, long nch)
    \brief dynamically creates a 2D array of integers
    \ingroup chapitre1
    \param nrl long, index on the first row element
    \param nrh long, index on the last row element
    \param ncl long, index on the first column element
    \param nch long, index on the last column element
    \return pointer on the array
  */
  int **imatrix(long nrl, long nrh, long ncl, long nch);
  /*!
    \fn void free_vector(int *v, long nl, long nh)
    \brief deletes an 1D array of integers dynamically created with ivector
    \ingroup chapitre1
    \param v int*, pointer on the array
    \param nl long, index on the first element
    \param nh long, index on the last element
    \sa ivector
  */
  void free_vector(int *v, long nl, long nh);
  /*!
    \fn void free_matrix(int **m, long nrl, long nrh, long ncl, long nch)
    \brief deletes a 2D array of integers dynamically created with imatrix
    \ingroup chapitre1
    \param m int**, pointer on the array
    \param nrl long, index on the first row element
    \param nrh long, index on the last row element
    \param ncl long, index on the first column element
    \param nch long, index on the last column element
    \sa imatrix
  */
  void free_matrix(int **m, long nrl, long nrh, long ncl, long nch);
  //@}
  //@{
  /** \name 1d/2d integer array manipulation functions. */
  /*!
    \fn void print_vector(int * b,long nl,long nh)
    \brief prints an 1D array of integers dynamically created with ivector
    \ingroup chapitre1
    \param b int*, pointer on the array
    \param nl long, index on the first element
    \param nh long, index on the last element
  */
  void print_vector(int * b,long nl,long nh);
  /*!
    \fn void print_matrix(int **b,long nrl,long nrh,long ncl,long nch)
    \brief prints a 2D array of integers dynamically created with imatrix
    \ingroup chapitre16
    \param b int**, pointer on the array
    \param nrl long, index on the first row element
    \param nrh long, index on the last row element
    \param ncl long, index on the first column element
    \param nch long, index on the last column element
  */
  void print_matrix(int **b,long nrl,long nrh,long ncl,long nch);
  /*!
    \fn void mult(int **mn,int *p,int *d,long n,long m)
    \brief multiply two 1d vectors.
    \ingroup chapitre16
    \param mn int**, pointer on the resulting matrix
    \param p int*, pointer on the first vector
    \param d int*, pointer on the second vector
    \param n long, length of vector p
    \param m long, length of vector d
  */
  void mult(int **mn,int *p,int *d,long n,long m);
  /*!
    \fn void mult_matrix(int **s1,int **s2,int **d,long nr,long nc,long ns)
    \brief multiply two 2d matrices.
    \ingroup chapitre16
    \param s1 int**, pointer on the first matrix
    \param s2 int**, pointer on the second matrix
    \param d int**, pointer on the resulting matrix
    \param nr long, number of lines in matrix s1
    \param nc long, number of columns in matrix s2
    \param ns long, number of columns (resp. lines) in matrix s1 (resp. s2).
  */
  void mult_matrix(int **s1,int **s2,int **d,long nr,long nc,long ns);
  /*!
    \fn void copy_vector(int *s,int *d,long nl,long nh)
    \brief copy of a 1d vector.
    \ingroup chapitre16
    \param s int*, pointer on the source vector
    \param d int*, pointer on the destination vector
    \param nl long, index on the first element
    \param nh long, index on the last element
  */
  void copy_vector(int *s,int *d,long nl,long nh);
  /*!
    \fn void copy_matrix(int **s,int **d,long nrl,long nrh,long ncl,long nch)
    \brief copy of a 2d matrix.
    \ingroup chapitre16
    \param s int**, pointer on the source matrix
    \param d int**, pointer on the destination matrix
    \param nrl long, index on the first row element
    \param nrh long, index on the last row element
    \param ncl long, index on the first column element
    \param nch long, index on the last column element
  */
  void copy_matrix(int **s,int **d,long nrl,long nrh,long ncl,long nch);
  //@}

#ifdef USENAMESPACE
} /* namespace NumericalRecipes */
#endif /* #ifdef USENAMESPACE */

#endif /* #ifdef _MATH_IMATH_H_ */
