#ifndef _TEMPLATE_BASICMATRIX_H_
#define _TEMPLATE_BASICMATRIX_H_

#include "Template/include.h"

/** \def MatrixErrorCode, the basic code of a matrix exception. */
#define MatrixErrorCode 70

/** \def MatrixErrorCode_MemoryNew, the code of an exception due to memory allocation in cMatrix class. */
#define MatrixErrorCode_MemoryNew MatrixErrorCode+1

/** \def MatrixErrorCode_MemoryFree, the code of an exception due to memory desallocation in cMatrix class. */
#define MatrixErrorCode_MemoryFree MatrixErrorCode+2

/** \def MatrixErrorCode_MemoryAccess, the code of an exception due to memory desallocation in cMatrix class.*/
#define MatrixErrorCode_MemoryAccess MatrixErrorCode+3

/** \def MatrixErrorCode_IndexOutOfBounds, the code of an exception due to errored index in the matrix. */
#define MatrixErrorCode_IndexOutOfBounds MatrixErrorCode+4

/** \def MatrixErrorCode_MatrixSize, the code of an exception due to errored index in the matrix. */
#define MatrixErrorCode_MatrixSize MatrixErrorCode+5

/** \def MatrixErrorCode_MatrixDefinition, the code of an exception due to errored matrix specification. */
#define MatrixErrorCode_MatrixDefinition MatrixErrorCode+6

template <int nr,int nc,class telt> class cBasicMatrix;
template <int nr,int nc,class telt> class cMatrix;
template <int nc,class telt> class cRowVector;
template <int nr,class telt> class cColumnVector;
template < int vectorLength, class vectorElement > class cVector;

/**
 * template base matrix container class.
 *
 * \param int nr, the width of the matrix.
 * \param int nc, the height of the matrix.
 * \param class telt, the type of contained elements.
 */
template <int nr = 1,int nc = 1,class telt> class cBasicMatrix
{
  private:
#define _typeBasicMatrix cBasicMatrix<nr,nc,telt>
  /**
   * utility function.
   * exchange two rows of a matrix.
   */
  void exchangeRows(const int& id1,const int& id2)
  {
    for (int j=0;j < nc;j++)
    {
      telt tmp = operator()(id1,j);
      operator()(id1,j) = operator()(id2,j);
      operator()(id2,j) = tmp;
    }
  }
  /**
   * utility function.
   * computes the pivot for partial pivoting method
   */
  int pivot (const int& row)
  {
    int k = row;
    telt amax,temp;
    amax = telt(-1);
    for (int i=row; i < nr; i++)
    {
      telt tmp = (operator()(i,row) < telt(0) ? -operator()(i,row) : operator()(i,row) );
      if ( ((temp = tmp) > amax) && (temp != telt(0)) )
	{
	  amax = temp;
	  k = i;
	}
      if (operator()(k,row) == telt(0))
	return -1;
      if (k != row)
      {
	exchangeRows(k,row);
	return k;
      }
    }
    return 0;
  }
  protected:
  /** pointer to matrix elements. */
  telt* _data;
  public:
  /**
   * the default constructor.
   * initialize a vector.
   */
  cBasicMatrix()
  {
    _data = new telt[nr*nc];
  }
  /**
   * the initialization constructor.
   * initialize a matrix with an array of elements.
   * \param const telt* data, the array of datas.
   */
  cBasicMatrix(const telt* data)
  {
    _data = new telt[nr*nc];
    int idx = 0;
    for (int i=0;i<nr;i++)
    {
      for (int j=0;j<nc;j++)
      {
	operator()(i,j) = data[idx]; idx++;
      }
    }
  }
  /**
   * the copy constructor.
   * initialize a matrix with another matrix.
   * \param const cBasicMatrix<nr,nc,telt>& m, the matrix to be copied
   */
  cBasicMatrix(const cBasicMatrix<nr,nc,telt>& m)
  {
    _data = new telt[nr*nc];
    for (int i=0;i<nr;i++)
    {
      for (int j=0;j<nc;j++)
      {
	operator()(i,j) = m(i,j);
      }
    }
  }
  /**
   * the destructor.
   */
  virtual ~cBasicMatrix()
  {
    if (_data != NULL)
    {
      delete [] _data;
    }
  }
  /**
   * access method to the height of a matrix.
   * \return a the height of a matrix.
   */
  int numberOfRows() const
  {
    return nr;
  }
  /**
   * access method to the width of a matrix.
   * \return a the width of a matrix.
   */
  int numberOfColumns() const
  {
    return nc;
  }
  /**
   * access method to a row of the matrix.
   * \param int idx, the index of the row.
   * \return a cRowVector object describing this row in the matrix.
   */
  virtual cRowVector<nc,telt> getRow(const int& idx)
  {
    if ((idx < 0) || (idx >= nr)) {}
    else {
      cRowVector<nc,telt> res(_data,idx,nc); return res;
    }
  }
  /**
   * access method to a column of the matrix.
   * \param int idx, the index of the column.
   * \return a cColumnVector object describing this column in the matrix.
   */
  virtual cColumnVector<nr,telt> getColumn(const int& idx)
  {
    if ((idx < 0) || (idx >= nc)) {}
    else {
      cColumnVector<nr,telt> res(_data,idx,nc); return res;
    }
  }
  /**
   * access method to an element of the matrix (as left member of an expression).
   * \param int idxr, the index of the row.
   * \param int idxc, the index of the column.
   * \return a reference on the element in the matrix.
   */
  virtual telt& operator()(const int& idxr,const int& idxc)
  {
    if ((idxr < 0) || (idxr >= nr)) {}
    else if ((idxc < 0) || (idxc >= nc)) {}
    else return _data[idxr*nc+idxc];
  }
  /**
   * access method to an element of the matrix (as right member of an expression).
   * \param int idxr, the index of the row.
   * \param int idxc, the index of the column.
   * \return a copy of the element in the matrix.
   */
  virtual telt operator()(const int& idxr,const int& idxc) const
  {
    if ((idxr < 0) || (idxr >= nr)) {}
    else if ((idxc < 0) || (idxc >= nc)) {}
    else return _data[idxr*nc+idxc];
  }
  /**
   * copy operator on matrix.
   * \param const cBasicMatrix<nr,nc,telt>& m, the matrix to be copied.
   * \return a reference on the copy of matrix lst.
   */
  cBasicMatrix<nr,nc,telt>& operator=(const cBasicMatrix<nr,nc,telt>& m)
  {
    if (&m != this)
    {
      for (int i=0;i<nr;i++)
      {
	for (int j=0;j<nc;j++)
	{
	  operator()(i,j) = m(i,j);
	}
      }
    }
    return *this;
  }  
  /**
   * computes identity matrix.
   * \return the nth identity matrix.
   */
  static cBasicMatrix<nr,nc,telt> getIdentity()
  {
    if (nr != nc) {}
    else {
      cBasicMatrix<nr,nc,telt> res;
      for (int i=0;i<nr;i++)
      {
	for (int j=0;j<nc;j++)
	  {
	    res(i,j) = (i == j) ? telt(1) : telt(0);
	  }
      }
    }
    return res;
  }
  /**
   * computes a random matrix.
   * \param const telt& m, the minimum value.
   * \param const telt& M, the maximum value.
   * \return the nr x nc random matrix (values are between m and M).
   */
  static cBasicMatrix<nr,nc,telt> getRandom(const telt& m,const telt& M)
  {
    cBasicMatrix<nr,nc,telt> res;
    for (int i=0;i<nr;i++)
    {
      for (int j=0;j<nc;j++)
	{
	  res(i,j) = m + ((M - m) * telt(rand()) / RAND_MAX);
	}
    }
    return res;
  }
  /**
   * utility function.
   * computes the inverse of a matrix.
   */
  cBasicMatrix<nc,nr,telt> inv()
  {
    cBasicMatrix<nr,nc,telt> temp = (*this);
    cBasicMatrix<nc,nr,telt> res = cBasicMatrix<nc,nr,telt>::getIdentity();
    size_t i,j,k;
    telt a1,a2;
    for (k=0; k < nr; k++)
    {
      int indx = temp.pivot(k);
      if (indx == -1) return;
      if (indx != 0)
	exchangeRows(k,indx);
      a1 = temp(k,k);
      for (j=0; j < nr; j++)
	{
	  temp(k,j) /= a1;
	  res(k,j) /= a1;
	}
      for (i=0; i < nr; i++)
	{
	  if (i != k)
	    {
	      a2 = temp(i,k);
	      for (j=0; j < nr; j++)
		{
		  temp(i,j) -= a2 * temp(k,j);
		  res(i,j) -= a2 * res(k,j);
		}
	    }
	}
    }
    return res;
  }
  /**
   * utility function.
   * computes the pseudo-inverse of a matrix.
   */
  cBasicMatrix<nc,nr,telt> pinv()
  {
    return (~(!(*this) * (*this)) * !(*this));
  }

  /**
   * unary + operator.
   * \return a copy of the reference matrix.
   */
  cBasicMatrix<nr,nc,telt> operator+()
  {
    cBasicMatrix<nr,nc,telt> res;
    res = (*this);
    return res;
  }

  /**
   * unary - operator.
   * \return the opposite of the reference matrix.
   */
  cBasicMatrix<nr,nc,telt> operator-()
  {
    cBasicMatrix<nr,nc,telt> res;
    for (int i=0;i<nr;i++)
    {
      for (int j=0;j<nc;j++)
	{
	  res(i,j) = -operator()(i,j);
	}
    }
    return res;
  }

  /**
   * unary ! operator.
   * \return the transpose of the reference matrix.
   */
  cBasicMatrix<nc,nr,telt> operator!()
  {
    cBasicMatrix<nc,nr,telt> res;
    for (int i=0;i<nr;i++)
    {
      for (int j=0;j<nc;j++)
	{
	  res(j,i) = operator()(i,j);
	}
    }
    return res;
  }

  /**
   * unary ~ operator.
   * \return the (pseudo-)inverse of the reference matrix.
   */
  cBasicMatrix<nc,nr,telt> operator~()
  {
    if (nr != nc) return pinv();
    else return inv();
  }

  /**
   * unary * operator.
   * \return the norm of the matrix.
   */
  telt operator*()
  {
    telt retVal = telt(0);
    for (int i=0; i < nr; i++)
    {
      for (int j=0; j < nc; j++)
	{
	  retVal += (operator()(i,j) * operator()(i,j));
	}
    }
    retVal = sqrt( retVal);
    return retVal;
  }

  /**
   * binary addition operator.
   */
  cBasicMatrix<nr,nc,telt> operator+(const cBasicMatrix<nr,nc,telt>& mr)
  {
    cBasicMatrix<nr,nc,telt> res;
    for (int i=0;i<nr;i++)
    {
      for (int j=0;j<nc;j++)
	{
	  res(i,j) = operator()(i,j) + mr(i,j);
	}
    }
    return res;
  }

  /**
   * binary substraction operator.
   */
  cBasicMatrix<nr,nc,telt> operator-(const cBasicMatrix<nr,nc,telt>& mr)
  {
    cBasicMatrix<nr,nc,telt> res;
    for (int i=0;i<nr;i++)
    {
      for (int j=0;j<nc;j++)
	{
	  res(i,j) = operator()(i,j) - mr(i,j);
	}
    }
    return res;
  }

  /**
   * binary matrix multiplication operator.
   */
  template <int m>  cBasicMatrix<nr,m,telt> operator*(const cBasicMatrix<nc,m,telt>& mr)
  {
    cBasicMatrix<nr,m,telt> res;
    for (int i=0;i<nr;i++)
    {
      for (int j=0;j<m;j++)
	{
	  res(i,j) = telt(0);
	  for (int k=0;k<nc;k++)
	    {
	      res(i,j) += operator()(i,k) * mr(k,j);
	    }
	}
    }
    return res;
  }

  /**
   * binary matrix multiplication operator.
   * specialize operator * with 1x1 matrix as right operand.
   */
  cBasicMatrix<nr,nc,telt> operator*(const cBasicMatrix<1,1,telt>& mr)
  {
    cBasicMatrix<nr,nc,telt> res;
    for (int i=0;i<nr;i++)
    {
      for (int j=0;j<nc;j++)
	{
	  res(i,j) =mr(0,0) * operator()(i,j);
	}
    }
    return res;
  }

  /**
   * binary scalar division operator.
   */
  cBasicMatrix<nr,nc,telt> operator/(const telt& v)
  {
    cBasicMatrix<nr,nc,telt> res;
    for (int i=0;i<nr;i++)
    {
      for (int j=0;j<nc;j++)
	{
	  res(i,j) = operator()(i,j) / v;
	}
    }
    return res;
  }

  /**
   * binary equality comparison operator.
   */
  bool operator==(const cBasicMatrix<nr,nc,telt>& mr)
  {
    bool res = true;
    for (int i=0;(i<nr) && res;i++)
    {
      for (int j=0;(j<nc) && res;j++)
	{
	  res = TEMPLATE_ABS((operator()(i,j) - mr(i,j)) <= TEMPLATE_TINY);
	}
    }
    return res;
  }

  /**
   * binary inequality comparison operator.
   */
  bool operator!=(const cBasicMatrix<nr,nc,telt>& mr)
  {
    bool res = true;
    for (int i=0;(i<nr) && res;i++)
    {
      for (int j=0;(j<nc) && res;j++)
	{
	  res = TEMPLATE_ABS((operator()(i,j) - mr(i,j)) > TEMPLATE_TINY);
	}
    }
    return res;
  }

  friend class cBasicMatrix<1,1,telt>;
  friend class cMatrix<nr,nc,telt>;
  friend class cMatrix<nc,nr,telt>;
  friend class cRowVector<nc,telt>;
  friend class cColumnVector<nr,telt>;
};

/**
 * stream dumping of a matrix.
 * dumps a matrix on a stream.
 * \param ostream& out, the dumping stream.
 * \param const const cBasicMatrix<nr,nc,telt>& v, the matrix to dump.
 * \return a reference on the dumping stream.
 */
template <int nr,int nc,class telt>  ostream& operator<<(ostream& out,const cBasicMatrix<nr,nc,telt>& m)
{
  for (int i=0;i<nr;i++)
    {
      for (int j=0;j<nc;j++)
	{
	  out << setprecision(10) << m(i,j);
	  if (j < nc -1) out << "  ";
	}
      if (i < nr-1) out << endl;
    }
  return out;
};

/**
 * binary matrix/telt multiplication operator.
 */
template <int nr,int nc,class telt> cBasicMatrix<nr,nc,telt> operator*(const cBasicMatrix<nr,nc,telt>& ml,const telt& v)
{
  cBasicMatrix<nr,nc,telt> res;
  for (int i=0;i<nr;i++)
    {
      for (int j=0;j<nc;j++)
	{
	  res(i,j) =v * ml(i,j);
	}
    }
  return res;
};

/**
 * binary telt/matrix multiplication operator.
 */
template <int nr,int nc,class telt> cBasicMatrix<nr,nc,telt> operator*(const telt& v,const cBasicMatrix<nr,nc,telt>& ml)
{
  cBasicMatrix<nr,nc,telt> res;
  for (int i=0;i<nr;i++)
    {
      for (int j=0;j<nc;j++)
	{
	  res(i,j) =v * ml(i,j);
	}
    }
  return res;
};

#endif
