#ifndef _TEMPLATE_COLUMNVECTOR_H_
#define _TEMPLATE_COLUMNVECTOR_H_

#include "Template/include.h"
#include "Template/BasicMatrix.h"

template <int nr,class telt> class cColumnVector : public cBasicMatrix<nr,1,telt>
{
 private:
  #define _typeColumnVector cColumnVector<nr,telt>
 protected:
  /** is the vector a reference on a matrix column ? */
  bool _ismatrixreference;
  /** which column */
  int _matrixcolumnindex;
  /* size of the referenced matrix */
  int _matrixsize;
 public:
  /**
   * the referencing constructor.
   * initialize a vector as a reference on a matrix column.
   * \param const telt* data, the matrix internal datas.
   * \param const int& ci, the matrix column index.
   * \param const int& ms, the matrix size.
   * \sa cBasicMatrix.
   */
  cColumnVector(const telt* data,const int& ci,const int& ms) :
    _ismatrixreference(true),
    _matrixcolumnindex(ci),
    _matrixsize(ms)
    {
      _data = data;
    }
  /**
   * the default constructor.
   */
  cColumnVector() :
    cBasicMatrix<nr,1,telt>(),
    _ismatrixreference(false),
    _matrixcolumnindex(-1),
    _matrixsize(-1)
    { }
  /**
   * the initializing constructor.
   * initialize a vector with an array of elements
   * \param const telt* data, the array of datas.
   */
  cColumnVector(const telt* data) :
    cBasicMatrix<nr,1,telt>(data),
    _ismatrixreference(false),
    _matrixcolumnindex(-1),
    _matrixsize(-1)
    { }
  /**
   * the copy constructor.
   * initialize a vector with a copy of a column aligned matrix.
   * \param const cBasicMatrix<nr,1,telt>& m, the column aligned matrix to copy.
   * \sa cBasicMatrix.
   */
  cColumnVector(const cBasicMatrix<nr,1,telt>& m) :
    _ismatrixreference(false),
    _matrixcolumnindex(-1),
    _matrixsize(-1)
    {
      _data = new telt[nr];
      for (int i=0;i<nr;i++)
	{
	  _data[i] = m._data[i];
	}
    }
  /**
   * the copy constructor.
   * initialize a vector with a copy of another cColumnVector.
   * \param const cColumnVector<nr,telt>& m, the vector to copy.
   */
  cColumnVector(const cColumnVector<nr,telt>& m) :
    _ismatrixreference(m._ismatrixreference),
    _matrixcolumnindex(m._matrixcolumnindex),
    _matrixsize(m._matrixsize)
    {
      if (_ismatrixreference)
	{
	  _data = m._data;
	}
      else
	{
	  _data = new telt[nr];
	  for (int i=0;i<nr;i++)
	    {
	      _data[i] = m._data[i];
	    }
	}
    }
  /**
   * the copy constructor.
   * initialize a vector with a copy of a cVector.
   * \param const cVector<nr,telt>& m, the vector to copy.
   * \sa cVector.
   */
  cColumnVector(cVector<nr,telt>& rv) :
    _ismatrixreference(false),
    _matrixcolumnindex(-1),
    _matrixsize(-1)
    {
      _data = new telt[nr];
      for (int i=0;i<nr;i++) _data[i] = rv[i];
    }
  /**
   * the destructor.
   */
  virtual ~cColumnVector()
    {
      if (_ismatrixreference) _data = (telt*)NULL;
    }
  /**
   * access method to an element of the vector (as left member of an expression).
   * simulates matrix element access.
   * \param int idxr, the index of the row.
   * \param int idxc, the index of the column.
   * \return a reference on the element in the vector.
   */
  virtual telt& operator()(const int& idxr,const int& idxc)
  {
    if ((idxr < 0) || (idxr >= nr)) {}
    else if ((idxc < 0) || (idxc >= 1)) {}
    else return _data[(_ismatrixreference ? (idxr*_matrixsize+_matrixcolumnindex) : idxr)];
  }
  /**
   * access method to an element of the vector (as right member of an expression).
   * simulates matrix element access.
   * \param int idxr, the index of the row.
   * \param int idxc, the index of the column.
   * \return a copy of the element in the vector.
   */
  virtual telt operator()(const int& idxr,const int& idxc) const
  {
    if ((idxr < 0) || (idxr >= nr)) {}
    else if ((idxc < 0) || (idxc >= 1)) {}
    else return _data[(_ismatrixreference ? (idxr*_matrixsize+_matrixcolumnindex) : idxr)];
  }
  /**
   * access method to an element of the vector (as left member of an expression).
   * \param int idx, the index of the element.
   * \return a reference on the element in the vector.
   */
  telt& operator[](const int& idx)
    {
      if ((idx < 0) || (idx >= nr)) {}
      else return _data[(_ismatrixreference ? (idx*_matrixsize+_matrixcolumnindex) : idx)];
    }
  /**
   * access method to an element of the vector (as right member of an expression).
   * \param int idx, the index of the element.
   * \return a copy of the element in the vector.
   */
  telt operator[](const int& idx) const
    {
      if ((idx < 0) || (idx >= nr)) {}
      else return _data[(_ismatrixreference ? (idx*_matrixsize+_matrixcolumnindex) : idx)];
    }
  /**
   * copy operator on vector.
   * \param const cColumnVector<nr,telt>& m, the vector to be copied.
   * \return a reference on the copy of vector m.
   */
  cColumnVector<nr,telt>& operator=(const cColumnVector<nr,telt>& m)
    {
      if (&m != this)
	{
	  for (int i=0;i<nr;i++)
	    {
	      _data[(_ismatrixreference ? (i*_matrixsize+_matrixcolumnindex) : i)] = m._data[(m._ismatrixreference ? (i*m._matrixsize+m._matrixcolumnindex) : i)];
	    }
	}
      return *this;
    }

  /**
   * unary ~ operator.
   * \return the normalized vector.
   */
  cColumnVector<nr,telt> operator~()
    {
      telt n = *(*this);
      return ((*this) / n);
    }

  friend class cRowVector<nr,telt>;
};

#endif
