#ifndef _TEMPLATE_LIST_H_
#define _TEMPLATE_LIST_H_

#include "Template/include.h"
#include "Template/Element.h"

/**
 * \enum ListPosition.
 * enumerated type defining various positions in a list.
 */
typedef enum {
  LP_first, /**< enum value LP_first, the beginning of the list. */
  LP_last, /**< enum value LP_last, the end of the list. */
  LP_pos /**< enum value LP_pos, an indexed position (use LP_pos+index). */
} ListPosition;

/** \def ListErrorCode, the basic code of a list exception. */
#define ListErrorCode 20
/** \def ListErrorCode_EmptyList, the code of an exception due to empty list access. */
#define ListErrorCode_EmptyList ListErrorCode+0
/** \def ListErrorCode_MemoryNew, the code of an exception due to memory allocation in cList class. */
#define ListErrorCode_MemoryNew ListErrorCode+1
/** \def ListErrorCode_MemoryFree, the code of an exception due to memory desallocation in cList class. */
#define ListErrorCode_MemoryFree ListErrorCode+2
/** \def ListErrorCode_InvalidIndex, the code of an exception due to errored index in the list. */
#define ListErrorCode_InvalidIndex ListErrorCode+3
/** \def ListErrorCode_RunningIteration, the code of an exception due to list iteration. */
#define ListErrorCode_RunningIteration ListErrorCode+4

template < class T > class cListIterator;

/**
 * template list container class.
 *
 * \param class T, the type of contained elements.
 */
template < class T > class cList {
private:
  /** the first element */
  cElement< T >* _head;
  /** the last element */
  cElement< T >* _tail;
  /** the number of elements in the list */
  int _card;
  /** is there an iteration on the list */
  int _isiterating;
protected:
  /**
   * removes all elements from the list.
   */
  void _removeall()
    {
      cElement< T >* tmp = _head->_next;
      while (tmp != _tail)
	{
	  cElement< T >* n = tmp->_next;
	  delete tmp;
	  _card--;
	  tmp = n;
	}
    }
public:
  /**
   * the constructor.
   * initialize a list.
   */
  cList() :
    _card(0) ,
    _isiterating(0)
    {
      _head = new cElement< T >(NULL,NULL);
      _tail = new cElement< T >(_head,NULL);
    }
  /**
   * the copy constructor.
   * initialize a list from another one.
   */
  cList(cList< T >& lst) :
    _card(0) ,
    _isiterating(0)
    {
      _head = new cElement< T >(NULL,NULL);
      _tail = new cElement< T >(_head,NULL);
      cElement< T >* tmp = lst._head->_next;
      while (tmp != lst._tail)
	{
	  cElement< T >* toadd = new cElement<T>(_tail->_prev,_tail,tmp->_data);
	  _card++;
	  tmp = tmp->_next;
	}
    }
  /**
   * the destructor.
   * removes a list from memory.
   */
  ~cList()
    {
      _removeall();
      delete _head;
      delete _tail;
    }
  /**
   * removes all elements of a list from memory.
   */
  void freeMemory()
    {
      _removeall();
    }
  /**
   * private attribute access method.
   * \return the number of elements contained in the list.
   */
  int card()
    {
      return _card;
    }
  /**
   * adds an element in the list.
   * \param T& v, the element to add.
   * \param ListPosition lp, the position where to add the element.
   * \return a reference on the current list.
   */
  cList< T >& add(T& v,ListPosition lp = LP_last)
    {
      if (_isiterating) {}
      else {
	cElement< T >* tmp;
	switch (lp)
	  {
	  case LP_first:
	    tmp = new cElement< T >(_head,_head->_next,v);
	    _card++;
	    break;
	  case LP_last:
	    tmp = new cElement< T >(_tail->_prev,_tail,v);
	    _card++;
	    break;
	  case LP_pos:
	    break;
	  default:
	    int idpos = lp - (int)LP_pos;
	    if (idpos < (_card+1))
	      {
		cElement< T >* tmpelt = _head->_next; int id = 0;
		while (id != idpos)
		  {
		    tmpelt = tmpelt->_next; id++;
		  }
		tmp = new cElement< T >(tmpelt->_prev,tmpelt,v);
	      }
	    else {}
	    _card++;
	  }
      }
      return *this;
    }
  /**
   * removes an element in the list.
   * \param T& v, the element to remove.
   * \return a reference on the current list.
   */
  cList< T >& del(T& v)
    {
      if (_isiterating) {}
      else {
	cElement< T >* tmp = _head->_next;
	while ((tmp != _tail) && (tmp->_data != v))
	  tmp = tmp->_next;
	if (!(tmp == _tail)) delete tmp;
	_card--;
      }
      return *this;
    }
  /**
   * access method to an element in the list.
   * \param int idx, the index of the element.
   * \return a reference on the element in the list.
   */
  T& operator[](int idx)
    {
      int id = 1;
      cElement< T >* tmp = _head->_next;
      while ((tmp != _tail) && (id != idx)) { tmp = tmp->_next; id++; }
      if (tmp == _tail) {}
      return tmp->_data;
    }
  /**
   * copy operator on lists.
   * if there is an iteration running on the current list, it does nothing.
   * \param cList< T>& lst, the list to be copied.
   * \return the copy of list lst.
   */
  cList< T >& operator =(cList< T >& lst)
    {
      if (_isiterating) return *this;
      if (this != &lst)
	{
	  _removeall();
	  cElement< T >* tmp = lst._head->_next;
	  while (tmp != lst._tail)
	    {
	      cElement< T >* toadd = new cElement< T >(_tail->_prev,_tail,tmp->_data); }
	      _card++;
	      tmp = tmp->_next;
	    }
	}
      return (*this);
    }
  /**
   * private attribute initialization method.
   * initialize the _isiterating attribute.
   * \param int& it, the value with which initialize _isiterating.
   */
  void iterate(int& it)
    {
      _isiterating = it;
    }
  /**
   * private attribute access method.
   * \return the value of _isiterating.
   */
  int iterate()
    {
      return _isiterating;
    }

  friend class cListIterator< T >;
  friend ostream& operator<<< T >(ostream& out,cList< T >& v);
};

/**
 * stream dumping of a list.
 * dumps a list on a stream.
 * \param ostream& out, the dumping stream.
 * \param cList< T >& v, the list to dump.
 * \return a reference on the dumping stream.
 */
template < class T > ostream& operator<<(ostream& out,cList< T >& v)
{
  int precis = out.precision();
  if (precis == 0)
    {
      out << "List(" << v.card() << ") [" << v._head->_prev << "," << v._head << "," << v._head->_next << "] [" << v._tail->_prev << "," << v._tail << "," << v._tail->_next << "]";
    }
  else
    {
      out << "(" << v.card() << ") [";
      for (cListIterator< T > iter = v;!(iter.finished());++iter)
	out << "\t" << iter.getCurrent() << endl;
      out << "]";
    }
  return out;
}

/**
 * adds an element at the end of a list.
 * \param cList< T >& lst, the list where adding is done.
 * \param T& v, the element to add.
 * \return a new list (copy of lst) where v has been added.
 */
template < class T > cList< T > operator+(cList< T >& lst,T& v)
{
  cList< T > res;
  res = lst;
  res.add(v,LP_last);
  return res;
};

/**
 * removes an element of a list.
 * \param cList< T >& lst, the list where removing is done.
 * \param T& v, the element to remove.
 * \return a new list (copy of lst) where v has been removed.
 */
template < class T > cList< T > operator-(cList< T >& lst,T& v)
{
  cList< T > res;
  res = lst;
  res.del(v);
  return res;
};

/**
 * combines two lists.
 * \param cList< T >& lst, the first list to concat.
 * \param cList< T >& lv, the second list to concat.
 * \return a new list,
 */
template < class T > cList< T > operator+(cList< T >& lst,cList< T >& lv)
{
  cList< T > res;
  res = lst;
  for (cListIterator< T > iter = lv;!(iter.finished());++iter)
    {
      res.add(iter.getCurrent(),LP_last);
    }
  return res;
};

/**
 * removes a list of elements of a list.
 * \param cList< T >& lst, the list where removing is done.
 * \param cList< T >& lv, the list containing elements to remove.
 * \return a new list (copy of lst) where elements of lv has been removed.
 */
template < class T > cList< T > operator-(cList< T >& lst,cList< T >& lv)
{
  cList< T > res;
  res = lst;
  for (cListIterator< T > iter = lv;!(iter.finished());++iter)
    {
      res.del(iter.getCurrent());
    }
  return res;
};

#endif
