#ifndef _TEMPLATE_MATRIX_H_
#define _TEMPLATE_MATRIX_H_

#include "Template/include.h"
#include "Template/BasicMatrix.h"

/**
 * template matrix container class.
 *
 * \param int nr, the width of the matrix.
 * \param int nc, the height of the matrix.
 * \param class telt, the type of contained elements.
 */
template <int nr,int nc,class telt> class cMatrix : public cBasicMatrix<nr,nc,telt>
{
 private:
  #define _typeMatrix cMatrix<nr,nc,telt>
 protected:
 public:
  /**
   * the default constructor.
   * initialize a vector.
   */
  cMatrix() :
    cBasicMatrix<nr,nc,telt>()
    { }
  /**
   * the initialization constructor.
   * initialize a matrix with an array of elements.
   * \param const telt* data, the array of datas.
   */
  cMatrix(const telt* data) :
    cBasicMatrix<nr,nc,telt>(data)
    { }
  /**
   * the copy constructor.
   * initialize a matrix with another matrix.
   * \param const cMatrix<nr,nc,telt>& m, the matrix to be copied
   */
  cMatrix(const cMatrix<nr,nc,telt>& m) :
    cBasicMatrix<nr,nc,telt>(m)
    { }
  /**
   * the destructor.
   */
  virtual ~cMatrix()
    {
      if (_data != NULL)
	{
	  delete [] _data;
	}
    }
  /**
   * access method to a row of the matrix.
   * \param int idx, the index of the row.
   * \return a cRowVector object describing this row in the matrix.
   */
  cRowVector<nc,telt> operator[](const int& idx) const
    {
      if ((idx < 0) || (idx >= nr)) {}
      else { cRowVector<nc,telt> res(_data,idx,nc); return res; }
    }
  /**
   * access method to a column of the matrix.
   * \param int idx, the index of the column.
   * \return a cColumnVector object describing this column in the matrix.
   */
  cColumnVector<nr,telt> operator()(const int& idx) const
    {
      if ((idx < 0) || (idx >= nc)) {}
      else { cColumnVector<nr,telt> res(_data,idx,nc); return res; }
    }
  /**
   * access method to an element of the matrix (as left member of an expression).
   * \param int idxr, the index of the row.
   * \param int idxc, the index of the column.
   * \return a reference on the element in the matrix.
   */
  virtual telt& operator()(const int& idxr,const int& idxc)
    {
      if ((idxr < 0) || (idxr >= nr)) {}
      else if ((idxc < 0) || (idxc >= nc)) {}
      else return _data[idxr*nc+idxc];
    }
  /**
   * access method to an element of the matrix (as right member of an expression).
   * \param int idxr, the index of the row.
   * \param int idxc, the index of the column.
   * \return a copy of the element in the matrix.
   */
  virtual telt operator()(const int& idxr,const int& idxc) const
    {
      if ((idxr < 0) || (idxr >= nr)) {}
      else if ((idxc < 0) || (idxc >= nc)) {}
      else return _data[idxr*nc+idxc];
    }
  /**
   * Numerical Recipes Library routine.
   * As described in "Numerical Recipes in c" pp. 46-47.
   * Given a matrix a[1..n][1..n], this routine returns the LU decomposition of a rowwise permutation of itself. a and n are input. a is output, arranged as in equation 2.3.14 (see NR); indx[1..n] is an output vector that records the row permutation effected by the partial pivoting; d is output as +/-1 depending on whether the number of row interchanges was even or odd, respectively. This routine is used in combination with lubksb to solve linear equations or invert a matrix.
   * \param indx returns the vector recording the row permutation.
   * \param d returns -1 or +1.
   * \return the LU decomposition of the reference matrix.
   * \sa lubksb.
  */
  cMatrix<nr,nc,telt> ludcmp(cColumnVector<nr,int>& indx,telt& d)
    {
      if (nr != nc) return;
      int i,imax,j,k;
      int n = nr;
      telt big,dum,sum,temp;
      cMatrix<nr,nc,telt> a(_data);
      cColumnVector<nr,telt> vv;
      d = telt(1);
      for (i=0;i<n;i++) {
	big = telt(0);
	for (j=0;j<n;j++)
	  if ((temp=TEMPLATE_ABS(a(i,j))) > big) big=temp;
	if (big == 0.) return;
	vv[i] = telt(1)/big;
      }
      for (j=0;j<n;j++) {
	for (i=0;i<j;i++) {
	  sum = a(i,j);
	  for (k=0;k<i;k++) sum -= a(i,k)*a(k,i);
	  a(i,j) = sum;
	}
	big = telt(0);
	for (i=j;i<n;i++) {
	  sum = a(i,j);
	  for (k=0;k<j;k++) sum -= a(i,k)*a(k,j);
	  a(i,j) = sum;
	  if ((dum=vv[i]*TEMPLATE_ABS(sum)) >= big) {
	    big = dum;
	    imax = i;
	  }
	}
	if (j != imax) {
	  for (k=0;k<n;k++) {
	    dum = a(imax,k);
	    a(imax,k) = a(j,k);
	    a(j,k) = dum;
	  }
	  d = -d;
	  vv[imax] = vv[j];
	}
	indx[j] = imax;
	if (a(j,j) == telt(0)) a(j,j) = telt(TEMPLATE_TINY);
	if (j != n-1) {
	  dum = telt(1)/(a(j,j));
	  for (i=j+1;i<n;i++) a(i,j) *= dum;
	}
      }
      return a;
    }
  /**
   * Numerical Recipes Library routine.
   * As described in "Numerical Recipes in c" pp. 47-48.
   * Solves the set of n linear equations A.X=B. Here a[1..n][1..n] is input, not as the matrix A but rather as its LU decomposition, determined by the routine ludcmp. indx[1..n] is input as the permutation vector returned by ludcmp. b[1..n] is input as the right-hand side vector B, and returns with the solution vector X. a, n, and indx are not modified by this routine and can be left in place for successive calls with different right-hand sides b. This routine takes into account the possibility that b begin with many zero elements, so it's efficient for use in matrix inversion.
   * \param indx is the permutation vector returned by ludcmp.
   * \param b is the input right-hand side vector.
   * \return the solution vector x.
   * \sa ludcmp.
   */
  cColumnVector<nr,telt> lubksb(cColumnVector<nr,int> indx,cColumnVector<nr,telt> b)
    {
      if (nr != nc) return;
      int i,ii=0,ip,j;
      int n = nr;
      telt sum;
      cColumnVector<nr,telt> x = b;
      for (i=0;i<n;i++) {
	ip = indx[i];
	sum = x[ip];
	x[ip] = x[i];
	if (ii >= 0)
	  for (j=ii;j<i-1;j++) sum -= operator()(i,j)*x[j];
	else if (sum) ii=i;
	x[i] = sum;
      }
      for (i=n-1;i>=0;i--) {
	sum = x[i];
	for (j=i+1;j<n;j++) sum -= operator()(i,j)*x[j];
	x[i] = sum/operator()(i,i);
      }
      return x;
    }
  /**
   * Numerical Recipes Library routine.
   * As described in "Numerical Recipes in c" pp. 67-70.
   * This routine computes the singular value decomposition, (*this) = U.W.V'. The matrix U is returned as an [1..n][1..m] matrix. The diagonal matrix of singular values W is output as a vector w[1..m]. The matrix V (not the transpose V') is output as v[1..m][1..m].
   * \param w, returns the vector describing W.
   * \param v, returns the matrix V.
   * \param ITMAX, maximum number of iterations (default is 30).
   * \return the matrix U.
   * \sa svbksb.
   */
  cMatrix<nr,nc,telt> svdcmp(cColumnVector<nc,telt>&w,cMatrix<nc,nc,telt>& v,const int& ITMAX = 30)
    {
      cMatrix<nr,nc,telt> a = (*this);
      int n = nc;
      int m = nr;
      int flag,i,its,j,jj,k,l,nm;
      telt anorm,c,f,g,h,s,scale,x,y,z;
      cColumnVector<nc,telt> rv1;
      g = scale = anorm = telt(0);
      for (i=0;i<n;i++)
	{
	  l = i+1;
	  rv1[i] = scale*g;
	  g = s = scale = telt(0);
	  if (i<m)
	    {
	      for (k=i;k<m;k++) scale += TEMPLATE_ABS(a(k,i));
	      if (scale > telt(0))
		{
		  for (k=i;k<m;k++)
		    {
		      a(k,i) /= scale;
		      s += a(k,i) * a(k,i);
		    }
		  f = a(i,i);
		  g = -TEMPLATE_SIGN(sqrt(s),f);
		  h = f*g-s;
		  a(i,i) = f-g;
		  for (j=l;j<n;j++)
		    {
		      for (s=0.0,k=i;k<m;k++) s += a(k,i)*a(k,j);
		      f = s/h;
		      for (k=i;k<m;k++) a(k,j) += f*a(k,i);
		    }
		  for (k=i;k<m;k++) a(k,i) *= scale;
		}
	    }

	  w[i] = scale*g;
	  g = s = scale = telt(0);
	  if ((i < m) && (i != n-1))
	    {
	      for (k=l;k<n;k++) scale += TEMPLATE_ABS(a(i,k));
	      if (scale)
		{
		  for (k=l;k<n;k++)
		    {
		      a(i,k) /= scale;
		      s += a(i,k)*a(i,k);
		    }
		  f = a(i,l);
		  g = -TEMPLATE_SIGN(sqrt(s),f);
		  h = f*g-s;
		  a(i,l) = f-g;
		  for (k=l;k<n;k++) rv1[k] = a(i,k)/h;
		  for (j=l;j<m;j++)
		    {
		      for (s=0.0,k=l;k<n;k++) s += a(j,k)*a(i,k);
		      for (k=l;k<n;k++) a(j,k) += s*rv1[k];
		    }
		  for (k=l;k<n;k++) a(i,k) *= scale;
		}
	    }
	  anorm = TEMPLATE_MAX(anorm,TEMPLATE_ABS(w[i])+TEMPLATE_ABS(rv1[i]));
	}
      for (i=n-1;i>=0;i--)
	{
	  if (i < n-1)
	    {
	      if (g)
		{
		  for (j=l;j<n;j++)
		    v(j,i) = (a(i,j)/a(i,l))/g;
		  for (j=l;j<n;j++)
		    {
		      for (s=0.0,k=l;k<n;k++) s += a(i,k)*v(k,j);
		      for (k=l;k<n;k++) v(k,j) += s*v(k,i);
		    }
		}
	      for (j=l;j<n;j++) v(i,j) = v(j,i) = telt(0);
	    }
	  v(i,i) = telt(1);
	  g = rv1[i];
	  l = i;
	}
      for (i=TEMPLATE_MIN(m,n)-1;i>=0;i--)
	{
	  l = i+1;
	  g = w[i];
	  for (j=l;j<n;j++) a(i,j) = telt(0);
	  if (g) {
	    g = telt(1)/g;
	    for (j=l;j<n;j++)
	      {
		for (s=0.0,k=l;k<m;k++) s += a(k,i)*a(k,j);
		f = (s/a(i,i))*g;
		for (k=i;k<m;k++) a(k,j) += f*a(k,i);
	      }
	    for (j=i;j<m;j++) a(j,i) *= g;
	  } else for (j=i;j<m;j++) a(j,i) = telt(0);
	  ++a(i,i);
	}
      for (k=n-1;k>=0;k--)
	{
	  for (its=1;its<=ITMAX;its++)
	    {
	      flag = 1;
	      for (l=k;l>=0;l--)
		{
		  nm = l-1;
		  if ((telt)(TEMPLATE_ABS(rv1[l])+anorm) == anorm)
		    {
		      flag = 0;
		      break;
		    }
		  if ((telt)(TEMPLATE_ABS(w[nm])+anorm) == anorm) break;
		}
	      if (flag)
		{
		  c = telt(0);
		  s = telt(1);
		  for (i=l;i<=k;i++)
		    {
		      f = s*rv1[i];
		      rv1[i] = c*rv1[i];
		      if ((telt)(TEMPLATE_ABS(f)+anorm) == anorm) break;
		      g = w[i];
		      h = TEMPLATE_pythag(f,g);
		      w[i] = h;
		      h = telt(1)/h;
		      c = g*h;
		      s = -f*h;
		      for (j=0;j<m;j++)
			{
			  y = a(j,nm);
			  z = a(j,i);
			  a(j,nm) = y*c+z*s;
			  a(j,i) = z*c-y*s;
			}
		    }
		}
	      z = w[k];
	      if (l == k)
		{
		  if (z < telt(0))
		    {
		      w[k] = -z;
		      for (j=0;j<n;j++) v(j,k) = -v(j,k);
		    }
		  break;
		}
	      if (its == ITMAX) return;
	      x = w[l];
	      nm = k-1;
	      y = w[nm];
	      g = rv1[nm];
	      h = rv1[k];
	      f = ((y-z)*(y+z)+(g-h)*(g+h))/(telt(2)*h*y);
	      g = TEMPLATE_pythag(f,telt(1));
	      f = ((x-z)*(x+z)+h*((y/(f+TEMPLATE_SIGN(g,f)))-h))/x;
	      c = s = telt(1);
	      for (j=l;j<=nm;j++)
		{
		  i = j+1;
		  g = rv1[i];
		  y = w[i];
		  h = s*g;
		  g = c*g;
		  z = TEMPLATE_pythag(f,h);
		  rv1[j] = z;
		  c = f/z;
		  s = h/z;
		  f = x*c+g*s;
		  g = g*c-x*s;
		  h = y*s;
		  y *= c;
		  for (jj=0;jj<n;jj++)
		    {
		      x = v(jj,j);
		      z = v(jj,i);
		      v(jj,j) = x*c+z*s;
		      v(jj,i) = z*c-x*s;
		    }
		  z = TEMPLATE_pythag(f,h);
		  w[j] = z;
		  if (z)
		    {
		      z = telt(1)/z;
		      c = f*z;
		      s = h*z;
		    }
		  f = c*g+s*y;
		  x = c*y-s*g;
		  for (jj=0;jj<m;jj++) {
		    y = a(jj,j);
		    z = a(jj,i);
		    a(jj,j) = y*c+z*s;
		    a(jj,i) = z*c-y*s;
		  }
		}
	      rv1[l] = telt(0);
	      rv1[k] = f;
	      w[k] = x;
	    }
	}
      return a;
    }
  /**
   * Numerical Recipes Library routine.
   * As described in "Numerical Recipes in c" p. 64.
   * Solves A.X=B for a vector X, where A is specified by the arrays u[1..m][1..n], w[1..n],v[1..n][1..n] as returned by svdcmp. m and n are the dimensions of a, and will be equal for square matrices. b[1..m] is the input right-hand side. x[1..n] is the output solution vector. Matrix U is given as the reference matrix. No input quantities are destroyed, so the routine may be called sequentially with different d's.
   * \param w is the vector representing the diagonal matrix of singular values W.
   * \param v is the matrix V (not V') from the SVD decomposition process.
   * \param b is the input right-hand side vector.
   * \return the solution vector.
   * \sa svdcmp.
   */
  cColumnVector<nc,telt> svbksb(const cColumnVector<nc,telt>& w,const cMatrix<nc,nc,telt>& v,const cColumnVector<nr,telt>& b)
    {
      int m = nr;
      int n = nc;
      int jj,j,i;
      telt s;
      cColumnVector<nc,telt> x;
      cColumnVector<nc,telt> tmp;
      for (j=0;j<n;j++)
	{
	  s = telt(0);
	  if (w[j])
	    {
	      for (i=0;i<m;i++) s += operator()(i,j)*b[i];
	      s /= w[j];
	    }
	  tmp[j] = s;
	}
      for (j=0;j<n;j++)
	{
	  s = telt(0);
	  for (jj=0;jj<n;jj++) s += v(j,jj)*tmp[jj];
	  x[j] = s;
	}
      return x;
    }

  friend class cBasicMatrix<nc,nr,telt>;
  friend class cMatrix<nc,nr,telt>;
  friend class cRowVector<nc,telt>;
  friend class cColumnVector<nr,telt>;
};

#endif
