#ifndef _TEMPLATE_ROWVECTOR_H_
#define _TEMPLATE_ROWVECTOR_H_

#include "Template/include.h"
#include "Template/BasicMatrix.h"

template <int nc,class telt> class cRowVector : public cBasicMatrix<1,nc,telt>
{
 private:
  #define _typeRowVector cRowVector<nc,telt>
 protected:
  /** is the vector a reference on a matrix row ? */
  bool _ismatrixreference;
  /** which row */
  int _matrixrowindex;
  /* size of the referenced matrix */
  int _matrixsize;
 public:
  /**
   * the referencing constructor.
   * initialize a vector as a reference on a matrix row.
   * \param const telt* data, the matrix internal datas.
   * \param const int& ri, the matrix row index.
   * \param const int& ms, the matrix size.
   * \sa cBasicMatrix.
   */
  cRowVector(const telt* data,const int& ri,const int& ms) :
    _ismatrixreference(true),
    _matrixrowindex(ri),
    _matrixsize(ms)
    {
      _data = data;
    }
  /**
   * the default constructor.
   */
  cRowVector() :
    cBasicMatrix<1,nc,telt>(),
    _ismatrixreference(false),
    _matrixrowindex(-1),
    _matrixsize(-1)
    { }
  /**
   * the initializing constructor.
   * initialize a vector with an array of elements
   * \param const telt* data, the array of datas.
   */
  cRowVector(const telt* data) :
    cBasicMatrix<1,nc,telt>(data),
    _ismatrixreference(false),
    _matrixrowindex(-1),
    _matrixsize(-1)
    { }
  /**
   * the copy constructor.
   * initialize a vector with a copy of a row aligned matrix.
   * \param const cBasicMatrix<1,nc,telt>& m, the row aligned matrix to copy.
   * \sa cBasicMatrix.
   */
  cRowVector(const cBasicMatrix<1,nc,telt>& m) :
    _ismatrixreference(false),
    _matrixrowindex(-1),
    _matrixsize(-1)
    {
      _data = new telt[nc];
      for (int i=0;i<nc;i++)
	{
	  _data[i] = m._data[i];
	}
    }
  /**
   * the copy constructor.
   * initialize a vector with a copy of another cRowVector.
   * \param const cRowVector<nr,telt>& m, the vector to copy.
   */
  cRowVector(const cRowVector<nc,telt>& m) :
    _ismatrixreference(m._ismatrixreference),
    _matrixrowindex(m._matrixrowindex),
    _matrixsize(m._matrixsize)
    {
      if (_ismatrixreference)
	{
	  _data = m._data;
	}
      else
	{
	  _data = new telt[nc];
	  for (int i=0;i<nc;i++)
	    {
	      _data[i] = m._data[i];
	    }
	}
    }
  /**
   * the copy constructor.
   * initialize a vector with a copy of a cVector.
   * \param const cVector<nr,telt>& m, the vector to copy.
   * \sa cVector.
   */
  cRowVector(cVector<nc,telt>& rv) :
    _ismatrixreference(false),
    _matrixrowindex(-1),
    _matrixsize(-1)
    {
      _data = new telt[nc];
      for (int i=0;i<nc;i++) _data[i] = rv[i];
    }
  /**
   * the destructor.
   */
  virtual ~cRowVector()
    {
      if (_ismatrixreference) _data = (telt*)NULL;
    }
  /**
   * access method to an element of the vector (as left member of an expression).
   * simulates matrix element access.
   * \param int idxr, the index of the row.
   * \param int idxc, the index of the column.
   * \return a reference on the element in the vector.
   */
  virtual telt& operator()(const int& idxr,const int& idxc)
  {
    if ((idxr < 0) || (idxr >= 1)) {}
    else if ((idxc < 0) || (idxc >= nc)) {}
    else return _data[(_ismatrixreference ? (_matrixrowindex*_matrixsize+idxc) : idxc)];
  }
  /**
   * access method to an element of the vector (as right member of an expression).
   * simulates matrix element access.
   * \param int idxr, the index of the row.
   * \param int idxc, the index of the column.
   * \return a copy of the element in the vector.
   */
  virtual telt operator()(const int& idxr,const int& idxc) const
  {
    if ((idxr < 0) || (idxr >= 1)) {}
    else if ((idxc < 0) || (idxc >= nc)) {}
    else return _data[(_ismatrixreference ? (_matrixrowindex*_matrixsize+idxc) : idxc)];
  }
  /**
   * access method to an element of the vector (as left member of an expression).
   * \param int idx, the index of the element.
   * \return a reference on the element in the vector.
   */
  telt& operator[](const int& idx)
    {
      if ((idx < 0) || (idx >= nc)) {}
      else { return _data[(_ismatrixreference ? (_matrixrowindex*_matrixsize+idx) : idx)]; }
    }
  /**
   * access method to an element of the vector (as right member of an expression).
   * \param int idx, the index of the element.
   * \return a copy of the element in the vector.
   */
  telt operator[](const int& idx) const
    {
      if ((idx < 0) || (idx >= nc)) {}
      else { return _data[(_ismatrixreference ? (_matrixrowindex*_matrixsize+idx) : idx)]; }
    }
  /**
   * copy operator on vector.
   * \param const cRowVector<nc,telt>& m, the vector to be copied.
   * \return a reference on the copy of vector m.
   */
  cRowVector<nc,telt>& operator=(const cRowVector<nc,telt>& m)
    {
      if (&m != this)
	{
	  for (int i=0;i<nc;i++)
	    {
	     _data[(_ismatrixreference ? (_matrixrowindex*_matrixsize+i) : i)] = m._data[(m._ismatrixreference ? (m._matrixrowindex*m._matrixsize+i) : i)];
	    }
	}
      return *this;
    }

  /**
   * unary ~ operator.
   * \return the normalized vector.
   */
  cRowVector<nc,telt> operator~()
    {
      telt n = *(*this);
      return ((*this) / n);
    }

  friend class cColumnVector<nc,telt>;
};

#endif
