#ifndef _UTILITY_BINARYCODING_H_
#define _UTILITY_BINARYCODING_H_

#include "Utility/Includes.h"

#ifdef USENAMESPACE
/*! \namespace BinaryCoder
 * Namespace containing coding/decoding functions.
 */
namespace BinaryCoder
{
#endif /* #ifdef USENAMESPACE */

/*! \class BinaryCoding
 * template class for binary encoding/decoding.
*/
template < class T > class BinaryCoding
{
 public:
  /*!
   * the maximum size of encoding.
   */
  static int codingSize;
  /*!
   * binary encoding of an object.
   * \param array unsigned char*, the buffer where encoding is done.
   * \param index long&, filling index in the buffer.
   * \param value T&, the value to encode.
   */
  static void encode(unsigned char* array,long& index,T& value);
  /*!
   * binary decoding of an object.
   * \param array unsigned char*, the buffer from where decoding is done.
   * \param index long&, reading index in the buffer.
   * \param value T&, the value to decode.
   */
  static void decode(unsigned char* array,long& index,T& value);
};

/*!
 * specialization of binary encoding/decoding for unsigned char.
*/
class BinaryCoding<unsigned char> {
public:
  static int codingSize;
  static void encode(unsigned char* array,long& index,unsigned char& value);
  static void decode(unsigned char* array,long& index,unsigned char& value);
};

/*!
 * specialization of binary encoding/decoding for unsigned short.
*/
class BinaryCoding<unsigned short> {
public:
  static int codingSize;
  static void encode(unsigned char* array,long& index,unsigned short& value);
  static void decode(unsigned char* array,long& index,unsigned short& value);
};

/*!
 * specialization of binary encoding/decoding for unsigned int.
*/
class BinaryCoding<unsigned int> {
public:
  static int codingSize;
  static void encode(unsigned char* array,long& index,unsigned int& value);
  static void decode(unsigned char* array,long& index,unsigned int& value);
};

/*!
 * specialization of binary encoding/decoding for unsigned long.
*/
class BinaryCoding<unsigned long> {
public:
  static int codingSize;
  static void encode(unsigned char* array,long& index,unsigned long& value);
  static void decode(unsigned char* array,long& index,unsigned long& value);
};

/*!
 * specialization of binary encoding/decoding for unsigned long long.
*/
class BinaryCoding<unsigned long long> {
public:
  static int codingSize;
  static void encode(unsigned char* array,long& index,unsigned long long& value);
  static void decode(unsigned char* array,long& index,unsigned long long& value);
};

/*!
 * specialization of binary encoding/decoding for char.
*/
class BinaryCoding<char> {
public:
  static int codingSize;
  static void encode(unsigned char* array,long& index,char& value);
  static void decode(unsigned char* array,long& index,char& value);
};

/*!
 * specialization of binary encoding/decoding for short.
*/
class BinaryCoding<short> {
public:
  static int codingSize;
  static void encode(unsigned char* array,long& index,short& value);
  static void decode(unsigned char* array,long& index,short& value);
};

/*!
 * specialization of binary encoding/decoding for int.
*/
class BinaryCoding<int> {
public:
  static int codingSize;
  static void encode(unsigned char* array,long& index,int& value);
  static void decode(unsigned char* array,long& index,int& value);
};

/*!
 * specialization of binary encoding/decoding for long.
*/
class BinaryCoding<long> {
public:
  static int codingSize;
  static void encode(unsigned char* array,long& index,long& value);
  static void decode(unsigned char* array,long& index,long& value);
};

/*!
 * specialization of binary encoding/decoding for long long.
*/
class BinaryCoding<long long> {
public:
  static int codingSize;
  static void encode(unsigned char* array,long& index,long long& value);
  static void decode(unsigned char* array,long& index,long long& value);
};

/*!
 * specialization of binary encoding/decoding for float.
*/
class BinaryCoding<float> {
public:
  static int codingSize;
  static void encode(unsigned char* array,long& index,float& value);
  static void decode(unsigned char* array,long& index,float& value);
};

/*!
 * specialization of binary encoding/decoding for double.
*/
class BinaryCoding<double> {
public:
  static int codingSize;
  static void encode(unsigned char* array,long& index,double& value);
  static void decode(unsigned char* array,long& index,double& value);
};

/*!
 * binary encoding of arrays.
 * \param array unsigned char*, the buffer where encoding is done.
 * \param index long&, filling index in the buffer.
 * \param value T*, the array of value to encode.
 * \param size long, the size of the array.
 */
template < class T > void encodeArray(unsigned char* array, long& index,T* value,long size)
{
  for (int i=0;i<size;i++) BinaryCoding<T>::encode(array,index,value[i]);
}

/*!
 * binary decoding of arrays.
 * \param array unsigned char*, the buffer from where decoding is done.
 * \param index long&, reading index in the buffer.
 * \param value T*, the array of value to decode.
 * \param size long, the size of the array.
 */
template < class T > void decodeArray(unsigned char* array, long& index,T* value,int size)
{
  for (int i=0;i<size;i++) BinaryCoding<T>::decode(array,index,value[i]);
}

#ifdef USENAMESPACE
} /* namespace BinaryCoder */
#endif /* #ifdef USENAMESPACE */

#ifdef USENAMESPACE
namespace Serializer
{
  using namespace BinaryCoder;
#endif /* #ifdef USENAMESPACE */
/*!
 * serialization of an object.
 * \param output ostream&, the stream where serialization is done.
 * \param value T&, the value to serialize.
 * \param binary int, kind of serialization (binary of text).
 */
template < class T > void serialize(ostream& output,T& value,int binary = 0)
{
  if (binary) {
    unsigned char array[BinaryCoding<T>::codingSize];
    long index = 0;
    BinaryCoding<T>::encode(array,index,value);
    output.write(array,index);
  } else {
    output << value << endl;
  }
}

/*!
 * unserialization of an object.
 * \param input istream&, the stream from where unserialization is done.
 * \param value T&, the value to unserialize.
 * \param binary int, kind of unserialization (binary of text).
 */
template < class T > void unserialize(istream& input,T& value,int binary = 0)
{
  if (binary) {
    unsigned char array[BinaryCoding<T>::codingSize];
    input.read(array,BinaryCoding<T>::codingSize);
    long index = 0;
    BinaryCoding<T>::decode(array,index,value);
  } else {
    input >> value;
  }
}

#ifdef USENAMESPACE
} /* namespace Serializer */
#endif /* #ifdef USENAMESPACE */

#endif /* #ifndef _UTILITY_BINARYCODING_H_ */
