#ifndef _UTILITY_COUNTERSTATISTICS_H_
#define _UTILITY_COUNTERSTATISTICS_H_

#include "Utility/Includes.h"
#include "Utility/Mode.h"

#ifdef USENAMESPACE
/*! \namespace CounterStatisticManager
 * Namespace containing counter statistics.
 */
namespace CounterStatisticManager
{
#endif /* #ifdef USENAMESPACE */
  /*! \class CounterInformation
   * class encapsulating informations about user defined counter.
   */
  class CounterInformation
    {
    private:
      /*!
       * name of the counter.
       */
      string _name;
      /*!
       * value of the counter.
       */
      long _value;
    protected:
    public:
      /*!
       * CounterInformation initialization constructor.
       * \param name string, the name of the counter.
       */
      CounterInformation(string name);
      /*!
       * CounterInformation destructor.
       */
      ~CounterInformation();
      /*!
       * CounterInformation member access method.
       * \return the name of the counter.
       */
      string getName();
      /*!
       * CounterInformation member access method.
       * \return the value of the counter.
       */
      long getValue();
      /*!
       * reset the value of the counter.
       */
      void reset();
      /*!
       * update the value of the counter.
       * \param value long, the vlue to add to the counter.
       */
      void update(long value);
      /*!
       * access function to the value of a counter.
       * \param name string, the name of the counter.
       * \return the value of the counter.
       */
      static long getValue(string name);
      /*!
       * CounterInformation dumping function.
       */
      static void counterStats();
    };
  /*! \var map < string , CounterInformation* > __counterlist
   * map containing all the registered counters.
   */
  extern map < string , CounterInformation* > __counterlist;
  /*! \typedef  map < string , CounterInformation* >::const_iterator __countersIterator
   * iterator on the map containing all the registered counters.
   */
  typedef  map < string , CounterInformation* >::const_iterator __countersIterator;
  /*! \fn void registerCounter(string name)
   * function to register a new counter.
   * \param name string, the name of the counter.
   */
  inline void registerCounter(string name)
  {
    if (isON(getCounterMode())) {
      if (__counterlist.find(name) == __counterlist.end())  new CounterInformation(name);
    }
  }
  /*! \fn void resetCounter(string name)
   * function to reset the value of a counter.
   * \param name string, the name of the counter.
   */
  inline void resetCounter(string name)
  {
    if (isON(getCounterMode())) {
      if (__counterlist.find(name) == __counterlist.end())  new CounterInformation(name);
      CounterInformation* _tmp = __counterlist[name];
      _tmp->reset();
    }
  }
  /*! \fn void updateCounter(string name,long delta)
   * function to update the value of a counter.
   * \param name string, the name of the counter.
   * \param delta long, the value to add to the counter.
   */
  inline void updateCounter(string name,long delta)
  {
    if (isON(getCounterMode())) {
      if (__counterlist.find(name) == __counterlist.end())  new CounterInformation(name);
      CounterInformation* _tmp = __counterlist[name];
      _tmp->update(delta);
    }
  }
  /*! \fn long getCounter(string name)
   * function to get the value of a counter.
   * \param name string, the name of the counter.
   */
  inline long getCounter(string name)
  {
    return (isON(getCounterMode()) ? CounterInformation::getValue(name) : 0);
  }
  /*! \fn ostream& dumpCounter(ostream& out,string name)
   * function to dump the value of a counter.
   * \param out ostream&, the stream where to dump the counter.
   * \param name string, the name of the counter.
   * \return a reference on the stream.
   */
  inline ostream& dumpCounter(ostream& out,string name)
  {
    return (out << name << " :: " << getCounter(name));
  }

#ifdef USENAMESPACE
} /* namespace CounterStatisticManager */
#endif /* #ifdef USENAMESPACE */

#endif /* #ifndef _UTILITY_COUNTERSTATISTICS_H_ */
