#ifndef _UTILITY_DEBUGGER_H_
#define _UTILITY_DEBUGGER_H_

#include "Utility/Includes.h"
#include "Utility/Mode.h"

#ifdef USENAMESPACE
/*! \namespace DebugManager
 * Namespace containing debugging functions.
 */
namespace DebugManager
{
#endif /* #ifdef USENAMESPACE */

  /*! \var ostream& consoleerror
   * Redirection of error messages.
   */
extern ostream& consoleerror;
  /*! \var ostream& consoledebug
   * Redirection of debug messages.
   */
extern ostream& consoledebug;
  /*! \var long decalagedebug
   * tabulate index of debug messages.
   */
extern long decalagedebug;
  /*! \fn void decalerdebug()
   * tabulator of debug messages.
   */
extern void decalerdebug();
  /*! \def ENTERINGFUNCTION
   * Macro to debug the start of a function.
   */
#define ENTERINGFUNCTION { \
if (isON(getDebugMode())) { \
decalerdebug(); \
consoledebug << "begin :: " << __FUNCTION__; \
decalagedebug++; \
consoledebug.flush(); \
} \
}
  /*! \def EXITINGFUNCTION
   * Macro to debug the end of a function.
   */
#define EXITINGFUNCTION { \
if (isON(getDebugMode())) \
  { decalagedebug--; decalerdebug(); consoledebug << "end :: " \
    << __FUNCTION__ << " :: " << __LINE__; consoledebug.flush(); } \
}
  /*! \def TRACEFUNCTION
   * Macro to trace an instruction in a function.
   */
#define TRACEFUNCTION { \
if (isON(getDebugMode())) \
  { decalerdebug(); consoledebug << "in :: " \
    << __FUNCTION__ << " :: " << __LINE__; consoledebug.flush(); } \
}
  /*! \def TRACECALL(v)
   * Macro to trace a call in a function.
   */
#define TRACECALL(v) { \
if (isON(getDebugMode())) \
  { TRACEFUNCTION; consoledebug << " call to " << #v; consoledebug.flush(); } \
}
  /*! \def TRACEVARIABLE(v)
   * Macro to trace the state of a variable in a function.
   */
#define TRACEVARIABLE(v) { \
if (isON(getDebugMode())) \
  { consoledebug << "  " << #v << "=[" << v << "]"; consoledebug.flush(); } \
}
  /*! \def TRACEMESSAGE(s)
   * Macro to dump a message.
   */
#define TRACEMESSAGE(s) { \
if (isON(getDebugMode())) \
  { consoledebug << s; consoledebug.flush(); } \
}
  /*! \def TRACEERROR(v)
   * Macro to trace an error.
   */
#define TRACEERROR(v) { \
consoleerror << endl << "Error in " << __FILE__ << "::" \
  << __FUNCTION__ << "[" << __LINE__ << "], error code = " << (v) << endl; throw (v); \
}
  /*! \def TRACEWARNING(v)
   * Macro to trace a warning.
   */
#define TRACEWARNING(v) { \
consoleerror << endl << "Warning in " << __FILE__ << "::" \
  << __FUNCTION__ << "[" << __LINE__ << "], warning code = " << (v) << endl; \
}

#ifdef USENAMESPACE
} /* namespace DebugManager */
#endif /* #ifdef USENAMESPACE */

#endif /* #ifndef _UTILITY_DEBUGGER_H_ */
