#ifndef _UTILITY_MEMORY_H_
#define _UTILITY_MEMORY_H_

#include "Utility/Includes.h"
#include "Utility/Mode.h"

#ifdef USENAMESPACE
/*! \namespace MemoryManager
 * Namespace containing memory allocation statistics.
 */
namespace MemoryManager
{
#endif /* #ifdef USENAMESPACE */

  /*! \def ALLOCATE(v,t)
   * simple memory allocator.
   */
#define ALLOCATE(v,t) { \
 (v) = new (t); if (isON(getMemoryMode())) registerMemory(#t,1); \
}
  /*! \def FREE(v,t)
   * simple memory desallocator.
   */
#define FREE(v,t) { \
 delete (v); if (isON(getMemoryMode())) unregisterMemory(#t,1); \
}
  /*! \def ALLOCATEARRAY(v,t,n)
   * simple array memory allocator.
   */
#define ALLOCATEARRAY(v,t,n) { \
 (v) = (t*)(new (t)[n]); if (isON(getMemoryMode())) registerMemory(#t,n); \
}
  /*! \def FREEARRAY(v,t,n)
   * simple array memory desallocator.
   */
#define FREEARRAY(v,t,n) { \
 delete [] (v); if (isON(getMemoryMode())) unregisterMemory(#t,n); \
}
  /*! \class MemoryAllocationInfo
   *
   */
class MemoryAllocationInfo
  {
  private:
      /*!
       * name of the type.
       */
    string _typename;
      /*!
       * total amount of allocated memory for this type.
       */
    long long _totalAllocated;
      /*!
       * maximum amount of allocated memory for this type, at the same time.
       */
    long long _maxAllocated;
      /*!
       * current amount of allocated memory for this type.
       */
    long long _numberAllocated;
  protected:
  public:
      /*!
       * MemoryAllocationInfo initialization constructor.
       * \param name string, the name of the type.
       * \param nbinit long long, the initial value.
       */
    MemoryAllocationInfo(string name,long long nbinit = 0);
       /*!
       * MemoryAllocationInfo destructor.
       */
    ~MemoryAllocationInfo();
      /*!
       * MemoryAllocationInfo member access method.
       * \return the name of the memory stat.
       */
      string getName();
      /*!
       * MemoryAllocationInfo member access method.
       * \return the total value of allocated memory.
       */
      long long getTotalValue();
      /*!
       * MemoryAllocationInfo member access method.
       * \return the maximum value of allocated memory.
       */
      long long getMaxValue();
      /*!
       * MemoryAllocationInfo member access method.
       * \return the current value of allocated memory.
       */
      long long getValue();
      /*!
       * reset the value of the memory stat.
       */
      void reset();
      /*!
       * update the value of the memory stat.
       * \param value long long, the vlue to add to the memory stat.
       */
      void update(long long value);
      /*!
       * access function to the total value of a memory stat.
       * \param name string, the name of the memory stat.
       * \return the total value of allocated memory.
       */
      static long long getTotalValue(string name);
      /*!
       * access function to the maximum value of a memory stat.
       * \param name string, the name of the memory stat.
       * \return the maximum value of allocated memory.
       */
      static long long getMaxValue(string name);
      /*!
       * access function to the current value of a memory stat.
       * \param name string, the name of the memory stat.
       * \return the current value of allocated memory.
       */
      static long long getValue(string name);
      /*!
       * MemoryAllocationInfo dumping function.
       */
      static void counterStats();
  };
  /*! \var map < string , MemoryAllocationInfo* > __memorylist
   * map containing all the registered memory allocation informations.
   */
  extern map < string , MemoryAllocationInfo* > __memorylist;
  /*! \typedef  map < string , MemoryAllocationInfo* >::const_iterator __memoryIterator
   * iterator on the map containing all the registered memory allocation informations.
   */
  typedef  map < string , MemoryAllocationInfo* >::const_iterator __memoryIterator;
  /*! \fn void registerMemory(string t,long long nb)
   * function to register an allocation of memory.
   * \param t string, the name of the memory stat.
   * \param nb long long, the amount of allocated memory.
   */
  inline void registerMemory(string t,long long nb)
  {
    if (isON(getMemoryMode())) {
      if (__memorylist.find(t) == __memorylist.end())  new MemoryAllocationInfo(t,0);
      MemoryAllocationInfo* _tmp = __memorylist[t];
      _tmp->update(nb);
    }
  }
  /*! \fn void unregisterMemory(string t,long long nb)
   * function to register a desallocation of memory.
   * \param t string, the name of the memory stat.
   * \param nb long long, the amount of allocated memory.
   */
  inline void unregisterMemory(string t,long long nb)
  {
    if (isON(getMemoryMode())) {
      if (__memorylist.find(t) == __memorylist.end())  new MemoryAllocationInfo(t);
      MemoryAllocationInfo* _tmp = __memorylist[t];
      _tmp->update(-nb);
    }
  }

#ifdef USENAMESPACE
} /* namespace MemoryManager */
#endif /* #ifdef USENAMESPACE */

#endif /* #ifndef _UTILITY_MEMORY_H_ */
