#include "radiosite/Ray.h"

#include "radiosite/VecLib.h"
#include "radiosite/Voxel.h"

long ray_count = 0;

int intersect_parallel_planes(double L,double R,double O,double D,double *tnear,double *tfar)
{
  if (D==0){
    if ((O<L)||(O>R)) return(0);
  }
  else{
    double t1=(L-O)/D,
      t2=(R-O)/D;
    if (t1 > t2) { 
      double t=t1;
      t1 = t2; t2 = t;
    }
    if (t1 > *tnear) *tnear = t1;
    if (t2 < *tfar) *tfar = t2;
    if (*tnear > *tfar) return(0);
    if (*tfar < 0) return(0);
  }
  return(1);
}

int HitBoundingBox(Point3 *minB,Point3 *maxB, Point3 *origin, Vector3 *dir,double *entrypoint,double *exitpoint)
{
  *entrypoint = -largeValue;
  *exitpoint = largeValue;
  if (intersect_parallel_planes(minB->x,maxB->x,
				origin->x,dir->x,
				entrypoint,exitpoint)==0) return(0);
  if (intersect_parallel_planes(minB->y,maxB->y,
				origin->y,dir->y,
				entrypoint,exitpoint)==0) return(0);
  if (intersect_parallel_planes(minB->z,maxB->z,
				origin->z,dir->z,
				entrypoint,exitpoint)==0) return(0);
  if (*entrypoint<0)*entrypoint=0;
  return(1);
}

int check_overlap(double min1,double max1,double min2,double max2)
{
  if (min1 > min2)
    {
      if (min1 > max2) return(0);
    }
  else if (min2 > max1) return(0);
  return(1);
}

int bounds_overlap(Box3 *b1,Box3 *b2)
{
  /* Check X extent */
  if (check_overlap(b1->min.x,b1->max.x,b2->min.x,b2->max.x))
    /* Check Y extent */
    if (check_overlap(b1->min.y,b1->max.y,b2->min.y,b2->max.y))
      /* Check Z extent */
      if (check_overlap(b1->min.z,b1->max.z,b2->min.z,b2->max.z))
	return(1);
  return(0);
}

double plane_intersect(Point3 *ray_start,Vector3 *ray_direction,Vector3 *plane_normal,double plane_constant)
{
  double t = -1.0,product;
  if(fabs(product=V3Dot(ray_direction,plane_normal))>smallEpsilon){
    if(product<0.0)
      t = -(plane_constant+V3Dot(ray_start,plane_normal))/product;
  }
  return(t);
}

double ray_quadrilateral(Point3 *ray_start,
			 Vector3 *ray_direction,
			 Vector3 *plane_normal, 
			 double plane_constant,
			 double Du0,double Du1,double Du2,
			 double Dux,double Duy,
			 double Dv0,double Dv1,double Dv2,
			 double Dvx,double Dvy,
			 Vector3 *Qux,Vector3 *Quy,
			 Vector3 *Qvx, Vector3 *Qvy,
			 Vector3 *Na,Vector3 *Nb,Vector3 *Nc,
			 double *pu,double *pv)
{
  double t=plane_intersect(ray_start,ray_direction,plane_normal,plane_constant);
  if (t > smallEpsilon){
    Point3 R;
    R.x = (*ray_start).x + t*(*ray_direction).x;
    R.y = (*ray_start).y + t*(*ray_direction).y;
    R.z = (*ray_start).z + t*(*ray_direction).z;
    if (fabs(Du2) < smallEpsilon)
      *pu = (V3Dot(Nc,&R)-Du0)/(Du1-V3Dot(Na,&R));
    else{
      double Ka=Dux+V3Dot(Qux,&R),
	Kb=Duy+V3Dot(Quy,&R);
      double discriminant;
      if ((Ka*Ka-Kb) < 0.0) 
	return(-1.0);
      else discriminant=sqrt(Ka*Ka-Kb);
      if (Ka < discriminant) *pu = Ka + discriminant;
      else *pu = Ka - discriminant;
    }
    if ( (*pu < 0.0) || (*pu > 1.0)) return(-1.0);
    if (fabs(Dv2) < smallEpsilon)
      *pv = (V3Dot(Nb,&R)-Dv0)/(Dv1-V3Dot(Na,&R));
    else{
      double Ka=Dvx+V3Dot(Qvx,&R),
	Kb=Dvy+V3Dot(Qvy,&R);
      double discriminant;
      if ((Ka*Ka-Kb) < 0.0) 
	return(-1.0);
      else discriminant=sqrt(Ka*Ka-Kb);
      if (discriminant < 0.0) return(-1.0);
      if (Ka < discriminant) *pv = Ka + discriminant;
      else *pv = Ka - discriminant;
    }
    if ( (*pv < 0.0) || (*pv > 1.0)) return(-1.0);
    return(t);
  }
  return(-1.);
}

double intersect_surface(Ray *ray,Surface *s,double *u,double *v)
{
  return(ray_quadrilateral(
			   &(ray->start),&(ray->direction),
			   &(s->normal), s->plane_constant,
			   s->Du0,s->Du1,s->Du2,s->Dux,s->Duy,
			   s->Dv0,s->Dv1,s->Dv2,s->Dvx,s->Dvy,
			   &(s->Qux),&(s->Quy),&(s->Qvx),
			   &(s->Qvy),&(s->Na),&(s->Nb),&(s->Nc),
			   u,v));
}

void get_ray_bound(Point3 *s,Vector3 *d,double t1,double t2,Box3 *b)
{
        Point3 p1,p2;
        point_on_line(*s,*d,t1,p1);
        point_on_line(*s,*d,t2,p2);
        b->min.x = MIN(p1.x,p2.x); b->max.x = MAX(p1.x,p2.x);
        b->min.y = MIN(p1.y,p2.y); b->max.y = MAX(p1.y,p2.y);
        b->min.z = MIN(p1.z,p2.z); b->max.z = MAX(p1.z,p2.z);
}

int get_nearest_surface_in_voxel(Ray *ray, double t_entry,double t_exit,Voxel *vox,double *min_t,double *pu,double *pv,Surface *slist)
{
  int i;
  int nearest_surface = UNDEFINED;
  Box3 ray_bound;
  double u=0.,v=0.;
  double t= -1.0;
  *min_t = largeValue;
  get_ray_bound(&(ray->start),&(ray->direction),t_entry,t_exit,&ray_bound);
  Surface *S= slist;
  for(i=0 ; i < vox->nsurfaces; i++) {
    int surface_num = vox->intersecting_surface_list[i];
    /* Enquire the mail box */
    if (S->mail_box.ray_num == ray->number){
      t = S->mail_box.t;
      u = S->mail_box.u;
      v = S->mail_box.v;
    }
    else if (bounds_overlap(&(S->bbox),&(ray_bound))){
      t  = intersect_surface(ray,S,&u,&v);
      S->mail_box.ray_num = ray->number;
      S->mail_box.t = t;
      S->mail_box.u = u;
      S->mail_box.v = v;
    }
    if (t > 0.0) /* If a valid intersection */
      if ((t_entry <= t) && (t_exit >= t))
	/*If the intersection is within bound*/
	if (t < *min_t){
	  nearest_surface = surface_num;
	  *min_t = t; *pu = u; *pv = v;
	}
    S = S->nexts;
  }
  return(nearest_surface);
}

int find_nearest_object_along_ray(Ray *ray,Surface *slist,double *t, double *u, double *v)
{
  double t_entry,t_exit;
  int nearest_surface = UNDEFINED;
  if(HitBoundingBox(&(volume_grid.extent.min),
		    &(volume_grid.extent.max),
		    &(ray->start),&(ray->direction),
		    &t_entry,&t_exit)){
    Vlist *vlist,*templist;
    vlist=templist=create_vlist(
				&(ray->start),
				&(ray->direction),
				t_entry,t_exit);
    while (templist!=NULL){
      Voxel *vox = volume_grid.voxels+templist->voxnum;
      if(vox->nsurfaces)
	if((nearest_surface=get_nearest_surface_in_voxel(
							 ray,templist->t_near,templist->t_far,
							 vox,t,u,v,slist))!=UNDEFINED) break;
      templist=templist->next;
    }
    purge_vlist(vlist);
  }
  return(nearest_surface);
}

int shadow_ray_test(Ray *ray,Surface *slist,double max_t)
{
  double t_entry,t_exit;
  int nearest_surface = UNDEFINED;
  double eps=0.00001,val;
  if(HitBoundingBox(&(volume_grid.extent.min),&(volume_grid.extent.max),&(ray->start),&(ray->direction),&t_entry,&t_exit))
    {
      Vlist *vlist,*templist;
      double t=-1,u,v;
      vlist=templist=create_vlist(&(ray->start),&(ray->direction),t_entry,max_t);
      while (templist!=NULL)
	{
	  Voxel *vox = volume_grid.voxels+templist->voxnum;
	  if(vox->nsurfaces)
	    if((nearest_surface=get_nearest_surface_in_voxel(ray,templist->t_near,templist->t_far,vox,&t,&u,&v,slist))!=UNDEFINED) break;
	  templist=templist->next;
	}
      purge_vlist(vlist);
      if ((max_t-t)>=0) val=(max_t-t);
      else val = t-max_t;
      if (val<eps) return(0);
      if (t > 0 && t < max_t) return(1);
      else return(0);
    }
  return(0);
}
