#include "radiosite/Shoot.h"

#include "radiosite/StaticMemory.h"
#include "radiosite/Lambda.h"
#include "radiosite/OpticalProperty.h"
#include "radiosite/Gauss.h"
#include "radiosite/Visibility.h"
#include "radiosite/Lumin.h"
#include "radiosite/Wavelet.h"
#include "radiosite/Link.h"

#include "recherche/Maillage.h"

#include "others/GlobalData.h"

#include "others/Debugger.h"

/**************************************************************************
 * Fonction d'initialisation de deltaB pour toutes les Surfaces de la
 * scene.
 *
 * ENTREE : l'environnenent,
 * SORTIE : void.
 */
void init_delta_B(Env* env)
{
  int i,k;

  Surface* S = env->slist;
  while (S != NULL) {
    for(k = 0; k < sampledSpectrumSize; k ++) {
      S->p.d_coeff[k][0] = S->E[k];
    }
    S = S->nexts;
  }

}/* fin init_delta_B */

/**************************************************************************
 * Fonction d'initialisation de deltaB pour toutes les Surfaces de la
 * scene. Ils sont mis a NULL.
 *
 * ENTREE : l'environnenent,
 * SORTIE : void.
 */
void init_null_delta_E(Patch *P)
{
  int k, j;
  
  if(P == NULL) return;
  
  for(k = 0; k < sampledSpectrumSize; k ++) {
    for(j = 0; j < __simulationParameters._waveletBasis * __simulationParameters._waveletBasis; j ++) {
      P->de_coeff[k][j] = 0.0;
    }
  }
}

/**************************************************************************
 * Fonction d'initialisation de deltaB pour toutes les Surfaces de la
 * scene. Ils sont mis a NULL.
 *
 * ENTREE : l'environnenent,
 * SORTIE : void.
 */
void init_null_delta_B(Patch *P)
{
  int k, j;
  
  if(P == NULL) return;
  
  for(k = 0; k < sampledSpectrumSize; k ++) {
    for(j = 0; j < __simulationParameters._waveletBasis * __simulationParameters._waveletBasis; j ++) {
      P->d_coeff[k][j] = 0.0;
    }
  }

  if(P->nextp == NULL) return;
  
  init_null_delta_B(&(P->nextp[0]));
  init_null_delta_B(&(P->nextp[1]));
  init_null_delta_B(&(P->nextp[2]));
  init_null_delta_B(&(P->nextp[3]));
}

/**************************************************************************
 * Fonction permettant d'evaluer la quantite d'energie a emettre pour
 * une surface.
 *
 * ENTREE : Le patch a traiter
 * SORTIE : La valeur totale du patch et de ses fils.
 */
double  evaluate_dB( Patch *P)
{
  double accu = 0.0;
  int j;
  
  for(j = 0; j < sampledSpectrumSize; j ++)           /* On calcule la quantite d'energie a emettre */
    accu += P->d_coeff[j][0];
  accu *= P->area;

  /* On calcule aussi celle des fils */
  if(P->nextp != NULL) {
    accu += evaluate_dB(P->nextp);
    accu += evaluate_dB(P->nextp + 1);
    accu += evaluate_dB(P->nextp + 2);
    accu += evaluate_dB(P->nextp + 3);
  }
  
  return accu;

}/* fin evaluate_dB */
  
/**********************************************************
 * Fonction permettant de determiner si un patch peut
 * emettre de l'energie.
 *
 * ENTREE : une surface;
 * SORTIE : un booleen.
 */
int reflector(Surface *s)
{
  int k;
  for(k=0;k<sampledSpectrumSize;k++)
    if (s->rho[k] > 0.)return(1);
  return(0);
}

/*****************************************************************
 * Fonction permettant de recuperer les surfaces ayant deja ete
 * liees a une source.
 *
 * ENTREE : le patch source,
 *          le tableau de surfaces liees, et leur nombre.
 * SORTIE : void.
 */
void get_seen_onestep(Patch *P, Surface** seen, int *nb_seen)
{
  Link *l;
  int i, found;
  l = P->link;
  //TRACEFUNCTION;
  while(l != NULL) {
    found = 0;
    /* On verifie que la surface n'est pas deja dans la liste */
    for(i = 0; (i < (*nb_seen)) && (!found) ; i++) {
      //TRACEFUNCTION; TRACEVARIABLE(i); TRACEVARIABLE(*nb_seen);
      if(seen[i] == l->source->s) found = 1;
    }

    /* Si elle n'y est pas, on l'ajoute */
    if(!found) {
      //TRACEMESSAGE("+");
      seen[*nb_seen] = l->source->s;
      (*nb_seen) ++;
    } else {
      //TRACEMESSAGE("-");
    }
    
    l = l->next_link;
  }
}

void get_seen(Patch *P, Surface** seen, int *nb_seen)
{
  //ENTERINGFUNCTION; TRACEVARIABLE(P); TRACEVARIABLE(*nb_seen); 

  if(P == NULL) return;
  get_seen_onestep(P,seen,nb_seen);

  if(P->nextp != NULL) {
    get_seen(&(P->nextp[0]), seen, nb_seen);
    get_seen(&(P->nextp[1]), seen, nb_seen);
    get_seen(&(P->nextp[2]), seen, nb_seen);
    get_seen(&(P->nextp[3]), seen, nb_seen);
  }
  //EXITINGFUNCTION;  
}/* fin get_seen */

/***********************************************************
 * Fonction de creation de liens entre une source et ses
 * surfaces visibles.
 *
 * ENTREE : Le vanishing moment, l'environnement,
 *          un seuil d'aire, un seuil de convergence,
 *          l'indice de la surface emitrice.
 *
 * SORTIE : VOID.
 */
void link_shoot_onestep(int M,Env *env,Surface* S,double Area_Threshold,double Threshold, Surface* source, Surface** seen, int *nb_seen)
{
  if(S->type_surface != TUNNELSURFACE_LIGHT) {
    if ((S != source) && (S->_highLevelParent != source->_highLevelParent)) {
      int vflag= visibility(env->slist,&(S->p),&(source->p));
      if (vflag != INVISIBLE) {
	if (S->_splitting) {
	  for (int i=0;i<S->_splitting;i++) {
	    link_shoot_onestep(M,env,S->_splitted[i],Area_Threshold,Threshold,source,seen,nb_seen);
	  }
	} else {
	  link(vflag,env->slist,M,0,&(S->p),0,&(source->p), Area_Threshold,Threshold);
	  seen[*nb_seen] = S;
	  (*nb_seen) = (*nb_seen) + 1;
	}
      }
    }
  }
}

void link_shoot(int M,Env *env,double Area_Threshold,double Threshold, Surface* source, Surface** seen, int *nb_seen)
{
  int i = 0;
  (*nb_seen) = 0;
  Surface* S = env->slist;
  while (S != NULL) {
    link_shoot_onestep(M,env,S,Area_Threshold,Threshold,source,seen,nb_seen);
    i++;
    S = S->nexts;
  }
}/* fin link */

Surface*  accelerated_shoot_find_source(Env *env, int step)
{
  int source = 0;
  int i, j, res=-1;
  double energie = 0.0;
  double accu;

  Surface* Ssou = NULL;

  Surface* S = env->slist;
  while (S != NULL) {

    /* Si la source n'a pas deja emis a cette iteration */
    if (S->already_done < step) {
      if ( S->o->emmit > 0.0)  {   /* Si le patch est une source */
	accu = 0.0;
	for(j = 0; j < sampledSpectrumSize; j ++)           /* On calcule la quantite d'energie a emettre */
	  accu += S->E[j];
	accu *= S->p.area;
	if(source == 0) { source = 1; energie = accu; Ssou = S; }
	else if(accu > energie) { energie = accu; Ssou = S; }
      }
      
      else if (source == 0) {              /* Si le patch n'est pas une source et qu'on n'a pas trouve de source */
	accu = 0.0;
	for (j = 0; j < sampledSpectrumSize; j ++)           /* On calcule la quantite d'energie a emettre */
	  accu += S->p.de_coeff[j][0];
	accu *= S->p.area;
	if (accu > energie) { energie = accu; Ssou = S; }
      }
    }
    S = S->nexts;
  }
  
  return Ssou;
}

/**************************************************************************
 * Fonction de recherche du patch a partir duquel on va shooter
 * l'energie.
 *
 * ENTREE : l'environnement
 * SORTIE : Le numero de la surface.
 */
Surface* findSource(Surface* S,int step,double& E)
{
  if (S->_splitting) {
    double EE = E;
    double Et = E; Surface* St;
    for (int i=0;i<S->_splitting;i++) {
      Surface* Stt = findSource(S->_splitted[i],step,Et);
      if (Et > EE) { EE = Et; St = Stt; }
    }
    E = EE;
    return St;
  } else {
    if( S->o->emmit > 0.0) {   /* Si le patch est une source */
      if(S->already_done < 1) { /* Si il n'a jamais emis */
	double accu = 0.0;
	for(int j = 0; j < sampledSpectrumSize; j ++)           /* On calcule la quantite d'energie a emettre */
	  accu += S->E[j];
	accu *= S->p.area;
	if (accu > E) { E = accu; return S; }
      }
    } else {              /* Si aucune source n'a encore ete choisie */
      double accu = 0.0;
      accu += evaluate_dB(&(S->p));
      if(accu > E) { E = accu; return S; }
    }
  }
  return NULL;
}

Surface* find_source(Env *env, int step)
{
  int source = 0;
  int i, j;
  Surface* res=NULL;
  double energie = 0.0;
  double accu;

  Surface* S = env->slist;
  while (S != NULL) {
    double energiet = 0.0;
    Surface* Sres = findSource(S,step,energiet);
    if (energiet > energie) { energie = energiet; res = Sres; }
    /*
      if(S->already_done < step) {
      if( S->o->emmit > 0.0) {
      if(S->already_done < 1) {
      accu = 0.0;
      for(j = 0; j < sampledSpectrumSize; j ++)
      accu += S->E[j];
      accu *= S->p.area;
      if(source == 0) { source = 1; energie = accu; res = S; }
      else if(accu > energie) { energie = accu; res = S; }
      }
      }
      else if(source == 0) {
      accu = 0.0;
      accu += evaluate_dB(&(S->p));
      if(accu > energie) { energie = accu; res = S; }
      }
      }
    */
    S = S->nexts;
  }

  return res;

}/* fin find_source */

/*******************************************************************
 * Fonction d'emission d'energie d'un patch P1 a un patch P2.
 * L'energie recue est transmise a la racine du patche recepteur
 * directement sans passer par un push-pull.
 *
 * ENTREE : le patch recepteur.
 * SORTIE : l'erreur.
 */
double shoot_energy(Patch *P, Link *l, int M, double Area_Threshold, double Threshold)
{
  int i,j,k,m1,m2,m,n1,n2,n,index;
  double error;
  double factor;
  int cpt;
  
  for(i = 0; i < sampledSpectrumSize; i ++)
    for(j = 0; j < M * M; j ++)
      StaticMemory::Shoot_shootenergy_C[i][j] = 0.0;
  
  for(factor = 1.0, cpt = P->no_patch / 10; cpt != 0; factor *= 0.5, cpt = cpt/10);

  /* Puis on collecte l'energie venant de la source */
  for(k=0;k<sampledSpectrumSize;k++){	
    for(m1=m=0;m1<M;m1++)
      for(m2=0;m2<M;m2++,m++) 
	for(n1=n=0;n1<M;n1++)
	  for(n2=0;n2<M;n2++,n++) 
	    StaticMemory::Shoot_shootenergy_C[k][m] +=P->s->rho[k]*l->F[m1][m2][n1][n2]* l->source->d_coeff[k][n];
    
    if (l->source->s->luminaire!=NULL && l->source->s->already_done == 0) {
      double s;
      
      for (i=index=0 ; i<M ; i++) 
	for (j=0 ; j<M ; j++, index++) {
	  s=eval_rad(k,P,l->source,gauss_quadrature_point[M-1][i],gauss_quadrature_point[M-1][j],Area_Threshold);
	  for(m1=m=0;m1<M;m1++)
	    for(m2=0;m2<M;m2++,m++) 
	      StaticMemory::Shoot_shootenergy_B[m] =P->s->rho[k]*l->F[m1][m2][0][0]*s/get_two_power_J(l->depth);		
	  StaticMemory::Shoot_shootenergy_f[index]=rebuild(M,StaticMemory::Shoot_shootenergy_B, i, j)/factor;
	  
	}
      
      for (i=index=0 ; i<M ; i++) 
	for (j=0 ; j<M ; j++, index++)
	  StaticMemory::Shoot_shootenergy_C[k][index]+=projection_coeff_2v(M,i+1,j+1,0,StaticMemory::Shoot_shootenergy_f)*factor;
    }
  }

  error = 0;
  for(k=0;k<sampledSpectrumSize;k++){
      
    if (fabs(StaticMemory::Shoot_shootenergy_C[k][0]) > error)
      error=fabs(StaticMemory::Shoot_shootenergy_C[k][0]);
      
    for(m = 0; m < M*M; m ++) {
      P->d_coeff[k][m] += StaticMemory::Shoot_shootenergy_C[k][m];
    }
  }
  
  /* Et on remonte l'energie C jusqu'a la racine */
  pull_up_energy_to_root(&(P->s->p), P->no_patch, M, StaticMemory::Shoot_shootenergy_C);
  
  return(error);
  
}/* fin shoot_energy */

/*******************************************************************
 * Fonction de shooting d'energie a partir d'un patch, vers
 * tous ses patches visibles et sans push-pull.
 *
 * ENTREE : La source, le patch visible de la source, etc.
 * SORTIE : VOID.
 */
void gather_energy_from_patch(Patch *S, Patch *P, int M, double Area_Threshold, double Threshold)
{
  Link *l;
  
  if(P == NULL) return;
  
  /* Pour chaque lien de P */
  for(l = P->link; l != NULL; l = l->next_link) {

    /* On recherche les liens vers la source */
    if(l->source->no_surface == S->no_surface) {
      /* On lui fait faire une collecte */
      shoot_energy(P, l, M, Area_Threshold, Threshold);
    }
  }

  if(P->nextp == NULL) return ;
  
  /* On shoot l'energie des fils de P */
  gather_energy_from_patch(S, &(P->nextp[0]), M, Area_Threshold, Threshold);
  gather_energy_from_patch(S, &(P->nextp[1]), M, Area_Threshold, Threshold);
  gather_energy_from_patch(S, &(P->nextp[2]), M, Area_Threshold, Threshold);
  gather_energy_from_patch(S, &(P->nextp[3]), M, Area_Threshold, Threshold);

}/* fin gather_energy_from_patch */

/*******************************************************************
 * Fonction de collecte de l'energie d'un patch p pour une source s.
 *
 * ENTREE : le vanishing moment, le facteur d'indice de profondeur du
 *          patch dans l'arbre, les coeffeicients de la fonction de radiosite,
 *          le seuil d'aire.
 * SORTIE : la difference entre l'ancienne radiosite et la nouvelle.
 */
double gather_from_source_with_push_pull(int M,double factor,Patch *p, Surface* source,
					 double **coeffs, double Area)
{

	/*
		Push-Pull algorithm used is similarly to
		the one proposed in Smits et al's error
		estimation paper.
		Basically its a Jacobi method.

		Error calculation is not very good.
		Currently I am finding L_infinity norm of
		the difference between the constant part
		of the function at (n-1)th iteration and n-th
		iteration.
	*/
  int i,j,k,m1,m2,m,n1,n2,n,index;
  double error;
  Link *l = p->link;
  
  while(l != NULL){

    if(l->source->s == source) {
      for(k=0;k<sampledSpectrumSize;k++){	
	for(m1=m=0;m1<M;m1++)
	  for(m2=0;m2<M;m2++,m++) 
	    for(n1=n=0;n1<M;n1++)
	      for(n2=0;n2<M;n2++,n++) {

		coeffs[k][m] += p->s->rho[k]*l->F[m1][m2][n1][n2]*l->source->d_coeff[k][n];
		
	      }
	
	if (l->source->s->luminaire!=NULL && l->source->s->already_done == 0) {
	  double s;

	  for (i=index=0 ; i<M ; i++) 
	    for (j=0 ; j<M ; j++, index++) {
	      s=eval_rad(k,p,l->source,gauss_quadrature_point[M-1][i],gauss_quadrature_point[M-1][j],Area);
	      for(m1=m=0;m1<M;m1++)
		for(m2=0;m2<M;m2++,m++) 
		  StaticMemory::Shoot_gatherfromsourcewithpushpull_B[m] =p->s->rho[k]*l->F[m1][m2][0][0]*s/get_two_power_J(l->depth);		
	      StaticMemory::Shoot_gatherfromsourcewithpushpull_f[index]=rebuild(M,StaticMemory::Shoot_gatherfromsourcewithpushpull_B, i, j)/factor;
	      
	    }
	  
	  for (i=index=0 ; i<M ; i++) 
	    for (j=0 ; j<M ; j++, index++)
	      coeffs[k][index]+=projection_coeff_2v(M,i+1,j+1,0,StaticMemory::Shoot_gatherfromsourcewithpushpull_f)*factor;
	}
      }
    }
    l = l->next_link;
  }
  
  if (p->nextp == NULL){
    error = 0;
    for(k=0;k<sampledSpectrumSize;k++){
      
      if (fabs(coeffs[k][0]) > error)
	error=fabs(coeffs[k][0]);
      
      for(m1=m=0;m1<M;m1++)
	for(m2=0;m2<M;m2++,m++) {
	  p->coeff[k][m] += coeffs[k][m];
	  p->d_coeff[k][m] += coeffs[k][m];
	}
      
      /*Not to take account of the emittance twice */	
    }
  }
  else {
    double temp_error;
    int i, j;
    
    error = 0;
    for(k=0;k<sampledSpectrumSize;k++)
      push_2v(M,coeffs[k],StaticMemory::Shoot_gatherfromsourcewithpushpull_c[0][k],StaticMemory::Shoot_gatherfromsourcewithpushpull_c[1][k],
	      StaticMemory::Shoot_gatherfromsourcewithpushpull_c[2][k],StaticMemory::Shoot_gatherfromsourcewithpushpull_c[3][k]);
    for(i=0;i<4;i++){
      temp_error=gather_from_source_with_push_pull(M,factor*0.5,p->nextp+i, source,StaticMemory::Shoot_gatherfromsourcewithpushpull_c[i], Area);
      if (temp_error > error) error = temp_error;
    }
    for(k=0;k<sampledSpectrumSize;k++)
      pull_2v(M,p->coeff[k],(p->nextp)->coeff[k],
	      (p->nextp+1)->coeff[k],
	      (p->nextp+2)->coeff[k],
	      (p->nextp+3)->coeff[k]);
  }
  
  return(error);

}/* fin gather_from_source_with_push_pull */

/*******************************************************************
 * Fonction permettant de shooter l'energie a partir
 * d'une source donnee. ATTENTION, pas de push_pull effectue...
 *
 */
double accelerated_shoot_from_source(int M,Env *env,double Area_Threshold,double Threshold, Surface* source, Surface** seen, int nb_seen)
{
  //ENTERINGFUNCTION;
  int i;
  Surface* index;
  double temp_error;
  double res = hugeValue;

  for(i = 0; i < nb_seen; i ++) {
    index = seen[i];
    //TRACEFUNCTION; TRACEVARIABLE(source); TRACEMESSAGE(" --> "); TRACEVARIABLE(index);
    /* On traville uniquement a partir des patches */
    gather_energy_from_patch(&(source->p), &(index->p), M, Area_Threshold, Threshold);
    //if (temp_error < res) res = temp_error;
  }

  /* on met already_done a jour */
  source->already_done ++;
  return res;
  //EXITINGFUNCTION;
}/* fin accelerated_shoot_from_source */

/*******************************************************************
 * Fonction permettant de shooter l'energie a partir
 * d'une source donnee.
 *
 */
double shoot_from_source_onestep(int M,Env *env,int id,double Area_Threshold,double Threshold, Surface* source, Surface** seen, int nb_seen)
{
  int k,j;
  Surface* index;
  double temp_error;
  double res = hugeValue;
 
  index = seen[id];    
  for(k=0;k<sampledSpectrumSize;k++) for(j=0;j<Parameters::_MaxM*Parameters::_MaxM;j++) StaticMemory::Shoot_shootfromsourceonestep_C[k][j]=0.0;
  if(index != source) {
    /* ne collecter l'energie que des sources "source" */
    temp_error = gather_from_source_with_push_pull(M, 1, &(index->p), source, StaticMemory::Shoot_shootfromsourceonestep_C, Area_Threshold);
    if (temp_error < res) res = temp_error;
  }
  return res;
}

double shoot_from_source(int M,Env *env,double Area_Threshold,double Threshold, Surface* source, Surface** seen, int nb_seen)
{

  int i;
  double temp_error;
  double res = hugeValue;

  /* Pour chaque face visible de la source */
  for(i = 0; i < nb_seen; i ++) {
    temp_error = shoot_from_source_onestep(M,env,i,Area_Threshold,Threshold,source,seen,nb_seen);
    if (temp_error < res) res = temp_error;
  }
  
  source->already_done ++;
  return res;
}/* fin shoot_from_source */

/****************************************************************
 * Fonction de push-pull de l'energie pour un patch.
 * En principe, le patch donne est la racine de l'arborescence
 * de la surface.
 *
 * ENTREE : le vanishing moment, un patch.
 * SORTIE : VOID.
 */
void push_pull_source(int M, Patch *P)
{
  int k,j;
  int MM;
  MM = M * M;


  /* Initialisation de d_coeff a la racine pour une source */
  if(P->s->luminaire!=NULL) {
    return;
  }

  /* Dans un fils : on met coeff a jour */
  if(P->nextp == NULL) {
    for(k = 0; k < sampledSpectrumSize; k++)
      for(j = 0; j < MM; j ++) {
	P->coeff[k][j] += P->d_coeff[k][j];
      }
    
    return;
  }

  /* PUSH */
  for(k = 0; k < sampledSpectrumSize; k++) {
    push_2v(M, P->d_coeff[k], StaticMemory::Shoot_pushpullsource_SW, StaticMemory::Shoot_pushpullsource_NW,
	    StaticMemory::Shoot_pushpullsource_SE, StaticMemory::Shoot_pushpullsource_NE);
    for(j = 0; j < MM ; j++) {
      P->nextp[0].d_coeff[k][j] += StaticMemory::Shoot_pushpullsource_SW[j];
      P->nextp[1].d_coeff[k][j] += StaticMemory::Shoot_pushpullsource_NW[j];
      P->nextp[2].d_coeff[k][j] += StaticMemory::Shoot_pushpullsource_SE[j];
      P->nextp[3].d_coeff[k][j] += StaticMemory::Shoot_pushpullsource_NE[j];
    }
  }

  /* On PUSH-PULL aux enfants */
  push_pull_source(M, &(P->nextp[0]));
  push_pull_source(M, &(P->nextp[1]));
  push_pull_source(M, &(P->nextp[2]));
  push_pull_source(M, &(P->nextp[3]));

  /* On remonte les valeurs des enfants */
  for(k = 0; k < sampledSpectrumSize; k++)
    pull_2v(M,P->d_coeff[k], P->nextp[0].d_coeff[k], P->nextp[1].d_coeff[k], P->nextp[2].d_coeff[k], P->nextp[3].d_coeff[k]);

  /* On modifie la radiosite du patch */
  for(k = 0; k < sampledSpectrumSize; k++)
    for(j = 0; j < MM; j ++)
      P->coeff[k][j] += P->d_coeff[k][j];
  
}/* fin push_pull_source */

/****************************************************************
 * Fonction d'affichage des coefficients d'un patch.
 * ENTREE : un patch.
 * SORTIE : void.
 */
void print_coeffs_source(Patch P)
{
  int j,k;

  printf("coefficients du pere : \n");

  for(k = 0; k < sampledSpectrumSize; k++)
    for(j = 0; j < __simulationParameters._waveletBasis*__simulationParameters._waveletBasis; j++)
      printf("k = %d   j = %d   coeff = %lf\n",k,j,P.coeff[k][j]);

  if(P.nextp == NULL) return;

  printf("coefficients du fils 1 : \n");
  print_coeffs_source(P.nextp[0]);
  printf("coefficients du fils 2 : \n");
  print_coeffs_source(P.nextp[1]);
  printf("coefficients du fils 3 : \n");
  print_coeffs_source(P.nextp[2]);
  printf("coefficients du fils 4 : \n");
  print_coeffs_source(P.nextp[3]);

}/* fin print_coeffs_source */

/****************************************************************
 * Fonction de shooting interactif : sauvegarde reguliere
 * des donnees.
 *
 * ENTREE :  Un Env, le seuil d'aire, le seuil de lien, le seuil d'erreur,
 *          Le vanishing moment, le nom du fichier rad a ecrire.
 * SORTIE : VOID.
 */
void shooter(Env* env, double Area_Threshold, double Threshold, double Error_Threshold,int MaxM)
{
  ENTERINGFUNCTION; TRACEVARIABLE(MaxM); TRACEVARIABLE(Error_Threshold); TRACEVARIABLE(Area_Threshold); TRACEVARIABLE(Threshold);

  Surface* Ssou;
      int i = 0;
    int step = 1;
    Surface** seen = NULL;
    int nb_seen;
     /* tableau des surfaces vues par la source choisie */
    seen = (Surface**)(new (Surface*)[env->N_surfaces]);

    init_delta_B(env);

    TRACEFUNCTION; TRACEVARIABLE(step);
    do {

      TRACEFUNCTION; TRACEVARIABLE(step); TRACEVARIABLE(i);

      if (interactiveMode) {
	Ssou = find_source(env, step);
      } else {
	Ssou = accelerated_shoot_find_source(env, step);
      }
      TRACEFUNCTION; TRACEVARIABLE(Ssou);
      nb_seen = 0;
      
      if(Ssou != NULL) {
	
	link_shoot(MaxM, env, Area_Threshold, Threshold, Ssou, seen, &nb_seen);

	if (interactiveMode) {
	  //shoot_from_source(MaxM,env,Area_Threshold,Threshold,i, seen, nb_seen);
	  init_null_delta_B(&(Ssou->p));
	  char fname[255]; sprintf(fname,"tempo_maillage_shoot_%d-%d.nff",step,i); saveNFF(fname,MaxM,env,ON,ON);
	} else {
	  push_pull_source(MaxM, &(Ssou->p));
	  accelerated_shoot_from_source(MaxM,env,Area_Threshold,Threshold,Ssou,seen, nb_seen);
	  init_null_delta_B(&(Ssou->p));
	  init_null_delta_E(&(Ssou->p));
	}
	i++;
	for (int i=0;i<nb_seen;i++) removeAllLinks(&(seen[i]->p));
      }

      else {
	TRACEFUNCTION; TRACEVARIABLE(step);
	if (interactiveMode) {
	  char fname[255]; sprintf(fname,"tempo_maillage_shoot_%d.nff",step); saveNFF(fname,MaxM,env,ON,ON);
	}
	i=0; step ++;
      }

      /*
      for(i = 0; i < env.N_surfaces; i++) {
	push_pull_source(MaxM, &(env.slist[i].p));
	init_null_delta_B(&(env.slist[i].p));
	init_null_delta_E(&(env.slist[i].p));
      }
      save_scene_rad(MaxM,env, FILE_OUT,Area_Threshold);
      call_visualizer();
      */

    } while(step < 5);

    if (!interactiveMode) {
      Surface* S = env->slist;
      while (S != NULL) {
	push_pull_source(MaxM, &(S->p));
	S = S->nexts;
      }
    }
    
    delete [] seen;
    EXITINGFUNCTION;
}/* fin non_interactive_shoot */

/*****************************************************************
 * Fonction de shooting
 *
 * ENTREE : Un Env, le seuil d'aire, le seuil de lien, le seuil d'erreur,
 *          Le vanishing moment, le nom du fichier rad a ecrire.
 * SORTIE : VOID.
 */
void Shoot(Env* env, double Area_Threshold, double Threshold, double Error_Threshold,int MaxM)
{
  ENTERINGFUNCTION;
  shooter(env, Area_Threshold, Threshold, Error_Threshold, MaxM);
  cout << "Apres shooting" << endl;
  Luminaire::stat(); Optical_prop::stat(); Link::stat(); Patch::stat();
  EXITINGFUNCTION;
}/* fin shoot */
