#include "radiosite/Surface.h"

#include "radiosite/Patch.h"
#include "radiosite/Link.h"
#include "radiosite/OpticalProperty.h"
#include "radiosite/Luminaire.h"
#include "radiosite/VecLib.h"
#include "radiosite/Kernel.h"
#include "radiosite/Lambda.h"
#include "radiosite/StaticMemory.h"

#define COPYPOINTMAILLAGEV(a,b) { a.x = b[0]; a.y = b[1]; a.z = b[2]; }
#define COPYPOINTMAILLAGE(a,b) { a.x = b.x; a.y = b.y; a.z = b.z; }

int Surface::_numberOfInstances = 0;

void Surface::stat()
{
  cout << "Surface::stat(" << sizeof(Surface) << ") :: " << _numberOfInstances << endl;
}

Surface::Surface()
{
  ALLOCATEARRAY(rho,double,sampledSpectrumSize);
  ALLOCATEARRAY(E,double,sampledSpectrumSize);
  o = NULL;
  nexts = NULL;
  luminaire = NULL;
  _splitting = SPLITTING_NONE;
  _splitted = NULL;
  _highLevelParent = NULL;
  _upperLevelParent = NULL;
  _numberOfInstances++;
}

Surface::Surface(Surface& s)
{
  o = NULL;
  nexts = NULL;
  luminaire = NULL;
  type_surface = s.type_surface;
  no_cellule = s.no_cellule;
  no_surface = s.no_surface;
  P1 = s.P1;
  P2 = s.P2;
  P3 = s.P3;
  P4 = s.P4;
  normal = s.normal;
  plane_constant = s.plane_constant;
  Na = s.Na;
  Nb = s.Nb;
  Nc = s.Nc;
  Qux = s.Qux;
  Quy = s.Quy;
  Qvx = s.Qvx;
  Qvy = s.Qvy;
  Du0 = s.Du0;
  Du1 = s.Du1;
  Du2 = s.Du2;
  Dux = s.Dux;
  Duy = s.Duy;
  Dv0 = s.Dv0;
  Dv1 = s.Dv1;
  Dv2 = s.Dv2;
  Dvx = s.Dvx;
  Dvy = s.Dvy;
  for (int i=0;i<4;i++) a[i] = s.a[i];
  ALLOCATEARRAY(rho,double,sampledSpectrumSize);
  ALLOCATEARRAY(E,double,sampledSpectrumSize);
  for (int i=0;i<sampledSpectrumSize;i++) rho[i] = s.rho[i];
  for (int i=0;i<sampledSpectrumSize;i++) E[i] = s.E[i];
  p = s.p; initSurface(p,this);
  mail_box.ray_num = s.mail_box.ray_num;
  mail_box.t = s.mail_box.t;
  mail_box.u = s.mail_box.u;
  mail_box.v = s.mail_box.v;
  bbox = s.bbox;
  if (s.o != NULL) o = new Optical_prop(*(s.o));
  if (s.luminaire != NULL) luminaire = new Luminaire(*(s.luminaire));
  already_done = s.already_done;
  _splitting = s._splitting;
  if (_splitting) {
    ALLOCATEARRAY(_splitted,Surface*,_splitting);
    for (int i=0;i<_splitting;i++) {
      ALLOCATEI(_splitted[i],Surface,*(s._splitted[i])); _splitted[i]->_upperLevelParent = this;
    }
  } else {
    _splitted = NULL;
  }
  _highLevelParent = NULL;
  _upperLevelParent = NULL;
  _numberOfInstances++;
}

Surface::Surface(enum eTunnelSurface type,double p0[3],double p1[3],double p2[3],double p3[3])
{
  o = NULL;
  nexts = NULL;
  luminaire = NULL;
  ALLOCATEARRAY(rho,double,sampledSpectrumSize);
  ALLOCATEARRAY(E,double,sampledSpectrumSize);

  o = new Optical_prop;
  o->refl=1.0;
  o->emmit=0.0;
  type_surface = (int)type;
  Patch *pp = &(p);
  pp->min_u = pp->min_v = 0.;
  pp->max_u = pp->max_v = 1.;
  COPYPOINTMAILLAGEV(pp->P1,p0);
  COPYPOINTMAILLAGE(P1,pp->P1);
  COPYPOINTMAILLAGEV(pp->P2,p1);
  COPYPOINTMAILLAGE(P2,pp->P2);
  COPYPOINTMAILLAGEV(pp->P3,p2);
  COPYPOINTMAILLAGE(P3,pp->P3);
  COPYPOINTMAILLAGEV(pp->P4,p3);
  COPYPOINTMAILLAGE(P4,pp->P4);
  area_related(&(P1),&(P2),&(P3),&(P4),a);
  Vector3 v0, v1, v2, v3;
  v1.x = (P2).x-(P1).x;
  v1.y = (P2).y-(P1).y;
  v1.z = (P2).z-(P1).z;
  v2.x = (P4).x-(P1).x;
  v2.y = (P4).y-(P1).y;
  v2.z = (P4).z-(P1).z;
  V3Cross(&v1,&v2,&(normal));
  V3Normalize(&(normal));   
  pp->area = 0.25*(a[0]+a[1]+a[2]+a[3]);
  pp->nextp = NULL;
  pp->link = NULL;
  plane_constant=-V3Dot(&(normal),&(pp->P1));
  v0.x=pp->P1.x-pp->P2.x+pp->P3.x-pp->P4.x;
  v0.y=pp->P1.y-pp->P2.y+pp->P3.y-pp->P4.y;
  v0.z=pp->P1.z-pp->P2.z+pp->P3.z-pp->P4.z;
  v3 = pp->P1;
  V3Cross(&v0,&(normal),&(Na));
  V3Cross(&v1,&(normal),&(Nb));
  V3Cross(&v2,&(normal),&(Nc));
  Du0 = V3Dot(&(Nc),&v3);
  Du1 = V3Dot(&(Na),&v3)+V3Dot(&(Nc),&v1);
  Du2 = V3Dot(&(Na),&v1);
  if (fabs(Du2) > smallEpsilon)
    {
      double frac = 1.0/Du2;
      Quy.x = -Nc.x*frac; 
      Quy.y = -Nc.y*frac; 
      Quy.z = -Nc.z*frac;
      Duy=Du0*frac;
      frac /= 2.0;
      Qux.x=Na.x*frac; 
      Qux.y=Na.y*frac; 
      Qux.z=Na.z*frac;
      Dux = -Du1*frac;
    }
  Dv0 = V3Dot(&(Nb),&v3);
  Dv1 = V3Dot(&(Na),&v3)+V3Dot(&(Nb),&v2);
  Dv2 = V3Dot(&(Na),&v2);
  if (fabs(Dv2) > smallEpsilon)
    {
      double frac = 1.0/Dv2;
      Qvy.x = -Nb.x*frac; 
      Qvy.y = -Nb.y*frac; 
      Qvy.z = -Nb.z*frac;
      Dvy=Dv0*frac;
      frac /= 2.0;
      Qvx.x=Na.x*frac; 
      Qvx.y=Na.y*frac; 
      Qvx.z=Na.z*frac;
      Dvx = -Dv1*frac;
    }
  _splitting = SPLITTING_NONE;
  _splitted = NULL;
  _highLevelParent = NULL;
  _upperLevelParent = NULL;
  _numberOfInstances++;
}

Surface::~Surface()
{
  FREEARRAY(rho,double,sampledSpectrumSize);
  FREEARRAY(E,double,sampledSpectrumSize);
  if (o != NULL) delete o;
  if (luminaire != NULL) delete luminaire;
  if (_splitting) {
    for (int i=0;i<_splitting;i++)
      FREE(_splitted[i],Surface*);
    FREEARRAY(_splitted,Surface*,_splitting);
  }
  _numberOfInstances--;
}

Surface& Surface::operator=(const Surface& ss)
{
  if (this != &ss)
    {
      if (o != NULL) delete o;
      if (luminaire != NULL) delete luminaire;
      o = NULL;
      nexts = NULL;
      luminaire = NULL;
      type_surface = ss.type_surface;
      no_cellule = ss.no_cellule;
      no_surface = ss.no_surface;
      P1 = ss.P1;
      P2 = ss.P2;
      P3 = ss.P3;
      P4 = ss.P4;
      normal = ss.normal;
      plane_constant = ss.plane_constant;
      Na = ss.Na;
      Nb = ss.Nb;
      Nc = ss.Nc;
      Qux = ss.Qux;
      Quy = ss.Quy;
      Qvx = ss.Qvx;
      Qvy = ss.Qvy;
      Du0 = ss.Du0;
      Du1 = ss.Du1;
      Du2 = ss.Du2;
      Dux = ss.Dux;
      Duy = ss.Duy;
      Dv0 = ss.Dv0;
      Dv1 = ss.Dv1;
      Dv2 = ss.Dv2;
      Dvx = ss.Dvx;
      Dvy = ss.Dvy;
      for (int i=0;i<4;i++) a[i] = ss.a[i];
      for (int i=0;i<sampledSpectrumSize;i++) rho[i] = ss.rho[i];
      for (int i=0;i<sampledSpectrumSize;i++) E[i] = ss.E[i];
      p = ss.p; initSurface(p,this);
      mail_box.ray_num = ss.mail_box.ray_num;
      mail_box.t = ss.mail_box.t;
      mail_box.u = ss.mail_box.u;
      mail_box.v = ss.mail_box.v;
      bbox = ss.bbox;
      if (ss.o != NULL) o = new Optical_prop(*(ss.o));
      if (ss.luminaire != NULL) luminaire = new Luminaire(*(ss.luminaire));
      already_done = ss.already_done;
      if (_splitting) {
	for (int i=0;i<_splitting;i++)
	  FREE(_splitted[i],Surface*);
	FREEARRAY(_splitted,Surface*,_splitting);
      }
      _splitting = ss._splitting;
      if (_splitting) {
	ALLOCATEARRAY(_splitted,Surface*,_splitting);
	for (int i=0;i<_splitting;i++) {
	  ALLOCATEI(_splitted[i],Surface,*(ss._splitted[i])); _splitted[i]->_upperLevelParent = this;
	}
      } else {
	_splitted = NULL;
      }
      _highLevelParent = NULL;
      _upperLevelParent = NULL;
    }
  return *this;
}
